@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,22 +25,29 @@ a distribution folder.
 """
 
 import os
-import shutil
-import subprocess
 import sys
-from logging import info, warning
 
+from nuitka.build.DataComposerInterface import runDataComposer
+from nuitka.constants.Serialization import ConstantAccessor
 from nuitka.finalizations.FinalizeMarkups import getImportedNames
+from nuitka.freezer.IncludedEntryPoints import (
+    addIncludedEntryPoints,
+    addShlibEntryPoint,
+    getStandardEntryPoints,
+    setMainEntryPoint,
+)
+from nuitka.freezer.Standalone import copyDataFiles
 from nuitka.importing import Importing, Recursion
 from nuitka.Options import getPythonFlags
 from nuitka.plugins.Plugins import Plugins
+from nuitka.PostProcessing import executePostProcessing
 from nuitka.PythonVersions import (
     getPythonABI,
     getSupportedPythonVersions,
-    isUninstalledPython,
     python_version,
-    python_version_str
+    python_version_str,
 )
+from nuitka.Tracing import general, inclusion_logger
 from nuitka.tree import SyntaxErrors
 from nuitka.utils import Execution, InstanceCounters, MemoryUsage, Utils
 from nuitka.utils.AppDirs import getCacheDir
@@ -49,21 +56,22 @@ from nuitka.utils.FileOperations import (
     hasFilenameExtension,
     listDir,
     makePath,
-    removeDirectory
+    removeDirectory,
 )
+from nuitka.utils.Importing import getSharedLibrarySuffix
+from nuitka.utils.ModuleNames import ModuleName
 
-from . import ModuleRegistry, Options, TreeXML
+from . import ModuleRegistry, Options, OutputDirectories, TreeXML
 from .build import SconsInterface
-from .codegen import CodeGeneration, ConstantCodes, Reports
+from .codegen import CodeGeneration, LoaderCodes, Reports
 from .finalizations import Finalization
-from .freezer.BytecodeModuleFreezer import generateBytecodeFrozenCode
-from .freezer.Standalone import copyUsedDLLs, detectEarlyImports
+from .freezer.Standalone import copyUsedDLLs
 from .optimizations import Optimization
 from .tree import Building
 
 
 def createNodeTree(filename):
-    """ Create a node tree.
+    """Create a node tree.
 
     Turn that source code into a node tree structure. If recursion into
     imported modules is available, more trees will be available during
@@ -73,79 +81,65 @@ def createNodeTree(filename):
 
     # First, build the raw node tree from the source code.
     main_module = Building.buildModuleTree(
-        filename = filename,
-        package  = None,
-        is_top   = True,
-        is_main  = not Options.shallMakeModule()
+        filename=filename,
+        package=None,
+        is_top=True,
+        is_main=not Options.shallMakeModule(),
     )
-    ModuleRegistry.addRootModule(main_module)
 
     # First remove old object files and old generated files, old binary or
     # module, and standalone mode program directory if any, they can only do
     # harm.
-    source_dir = getSourceDirectoryPath(main_module)
+    source_dir = OutputDirectories.getSourceDirectoryPath()
 
     if not Options.shallOnlyExecCCompilerCall():
         cleanSourceDirectory(source_dir)
 
     # Prepare the ".dist" directory, throwing away what was there before.
     if Options.isStandaloneMode():
-        standalone_dir = getStandaloneDirectoryPath(main_module)
-        removeDirectory(
-            path          = standalone_dir,
-            ignore_errors = True
-        )
+        standalone_dir = OutputDirectories.getStandaloneDirectoryPath()
+        removeDirectory(path=standalone_dir, ignore_errors=True)
         makePath(standalone_dir)
 
-    deleteFile(
-        path       = getResultFullpath(main_module),
-        must_exist = False
-    )
+    deleteFile(path=OutputDirectories.getResultFullpath(), must_exist=False)
 
     # Second, do it for the directories given.
     for plugin_filename in Options.getShallFollowExtra():
-        Recursion.checkPluginPath(
-            plugin_filename = plugin_filename,
-            module_package  = None
-        )
+        Recursion.checkPluginPath(plugin_filename=plugin_filename, module_package=None)
 
     for pattern in Options.getShallFollowExtraFilePatterns():
-        Recursion.checkPluginFilenamePattern(
-            pattern = pattern
-        )
+        Recursion.checkPluginFilenamePattern(pattern=pattern)
 
     for package_name in Options.getMustIncludePackages():
         package_package, package_directory, kind = Importing.findModule(
-            importing      = None,
-            module_name    = package_name,
-            parent_package = None,
-            level          = 0,
-            warn           = False
+            importing=None,
+            module_name=ModuleName(package_name),
+            parent_package=None,
+            level=0,
+            warn=False,
         )
 
         if kind != "absolute":
             sys.exit("Error, failed to locate package %r." % package_name)
 
         Recursion.checkPluginPath(
-            plugin_filename = package_directory,
-            module_package  = package_package
+            plugin_filename=package_directory, module_package=package_package
         )
 
     for module_name in Options.getMustIncludeModules():
         module_package, module_filename, kind = Importing.findModule(
-            importing      = None,
-            module_name    = module_name,
-            parent_package = None,
-            level          = 0,
-            warn           = False
+            importing=None,
+            module_name=ModuleName(module_name),
+            parent_package=None,
+            level=0,
+            warn=False,
         )
 
         if kind != "absolute":
             sys.exit("Error, failed to locate module %r." % module_name)
 
         Recursion.checkPluginSinglePath(
-            plugin_filename = module_filename,
-            module_package  = module_package
+            plugin_filename=module_filename, module_package=module_package
         )
 
     # Then optimize the tree and potentially recursed modules.
@@ -167,134 +161,112 @@ def dumpTreeXML(tree):
     TreeXML.dump(xml_root)
 
 
-def getTreeFilenameWithSuffix(tree, suffix):
-    return tree.getOutputFilename() + suffix
-
-
-def getSourceDirectoryPath(main_module):
-    assert main_module.isCompiledPythonModule()
-
-    return Options.getOutputPath(
-        path = os.path.basename(
-            getTreeFilenameWithSuffix(main_module, ".build")
-        )
-    )
-
-
-def getStandaloneDirectoryPath(main_module):
-    return Options.getOutputPath(
-        path = os.path.basename(
-            getTreeFilenameWithSuffix(main_module, ".dist")
-        )
-    )
-
-
-def getResultBasepath(main_module):
-    assert main_module.isCompiledPythonModule()
-
-    if Options.isStandaloneMode():
-        return os.path.join(
-            getStandaloneDirectoryPath(main_module),
-            os.path.basename(
-                getTreeFilenameWithSuffix(main_module, "")
-            )
-        )
-    else:
-        return Options.getOutputPath(
-            path = os.path.basename(
-                getTreeFilenameWithSuffix(main_module, "")
-            )
-        )
-
-
-def getResultFullpath(main_module):
-    """ Get the final output binary result full path.
-
-    """
-
-    result = getResultBasepath(main_module)
-
-    if Options.shallMakeModule():
-        result += Utils.getSharedLibrarySuffix()
-    else:
-        if Options.getOutputFilename() is not None:
-            result = Options.getOutputFilename()
-        elif Utils.getOS() == "Windows":
-            result += ".exe"
-        elif not Options.isStandaloneMode():
-            result += ".bin"
-
-    return result
-
-
 def cleanSourceDirectory(source_dir):
     extensions = (
-        ".bin", ".c", ".cpp", ".exp", ".h",
-        ".lib", ".manifest", ".o", ".obj",
-        ".os", ".rc", ".res", ".S", ".txt"
+        ".bin",
+        ".c",
+        ".cpp",
+        ".exp",
+        ".h",
+        ".lib",
+        ".manifest",
+        ".o",
+        ".obj",
+        ".os",
+        ".rc",
+        ".res",
+        ".S",
+        ".txt",
+        ".const",
     )
 
+    def check(path):
+        if hasFilenameExtension(path, extensions):
+            deleteFile(path, must_exist=True)
+
     if os.path.isdir(source_dir):
         for path, _filename in listDir(source_dir):
-            if hasFilenameExtension(path, extensions):
-                deleteFile(path, must_exist = True)
-    else:
-        makePath(source_dir)
+            check(path)
+
+        static_dir = os.path.join(source_dir, "static_src")
+
+        if os.path.exists(static_dir):
+            for path, _filename in listDir(static_dir):
+                check(path)
+
+        plugins_dir = os.path.join(source_dir, "plugins")
+
+        if os.path.exists(plugins_dir):
+            for path, _filename in listDir(plugins_dir):
+                check(path)
 
 
 def pickSourceFilenames(source_dir, modules):
+    """Pick the names for the C files of each module.
+
+    Args:
+        source_dir - the directory to put the module sources will be put into
+        modules    - all the modules to build.
+
+    Returns:
+        Dictionary mapping modules to filenames in source_dir.
+
+    Notes:
+        These filenames can collide, due to e.g. mixed case usage, or there
+        being duplicate copies, e.g. a package named the same as the main
+        binary.
+
+        Conflicts are resolved by appending @<number> with a count in the
+        list of sorted modules. We try to be reproducible here, so we get
+        still good caching for external tools.
+    """
+
     collision_filenames = set()
-    seen_filenames = set()
 
-    # Our output.
-    module_filenames = {}
-
-    def getFilenames(module):
-        base_filename =  os.path.join(
-            source_dir,
-            "module." + module.getFullName()
-              if not module.isInternalModule() else
-            module.getFullName()
-        )
+    def _getModuleFilenames(module):
+        base_filename = os.path.join(source_dir, "module." + module.getFullName())
 
         # Note: Could detect if the file system is cases sensitive in source_dir
         # or not, but that's probably not worth the effort. False positives do
-        # no harm at all.
-        collision_filename = os.path.normcase(base_filename)
+        # no harm at all. We cannot use normcase, as macOS is not using one that
+        # will tell us the truth.
+        collision_filename = base_filename.lower()
 
         return base_filename, collision_filename
 
+    seen_filenames = set()
+
     # First pass, check for collisions.
     for module in modules:
         if module.isPythonShlibModule():
             continue
 
-        base_filename = base_filename, collision_filename = getFilenames(module)
+        _base_filename, collision_filename = _getModuleFilenames(module)
 
         if collision_filename in seen_filenames:
             collision_filenames.add(collision_filename)
 
         seen_filenames.add(collision_filename)
 
-    # Count up for colliding filenames.
+    # Our output.
+    module_filenames = {}
+
+    # Count up for colliding filenames as we go.
     collision_counts = {}
 
     # Second pass, this time sorted, so we get deterministic results. We will
-    # apply an @1/@2 to disambiguate the filenames.
-    for module in sorted(modules, key = lambda x : x.getFullName()):
+    # apply an "@1"/"@2",... to disambiguate the filenames.
+    for module in sorted(modules, key=lambda x: x.getFullName()):
         if module.isPythonShlibModule():
             continue
 
-        base_filename = base_filename, collision_filename = getFilenames(module)
+        base_filename, collision_filename = _getModuleFilenames(module)
 
         if collision_filename in collision_filenames:
-            collision_counts[ collision_filename ] = \
-              collision_counts.get(collision_filename, 0) + 1
-            hash_suffix = "@%d" % collision_counts[ collision_filename ]
-        else:
-            hash_suffix = ""
-
-        base_filename += hash_suffix
+            collision_counts[collision_filename] = (
+                collision_counts.get(collision_filename, 0) + 1
+            )
+            base_filename += "@%d" % collision_counts[collision_filename]
 
         module_filenames[module] = base_filename + ".c"
 
@@ -305,17 +277,12 @@ standalone_entry_points = []
 
 
 def makeSourceDirectory(main_module):
-    """ Get the full list of modules imported, create code for all of them.
-
-    """
+    """Get the full list of modules imported, create code for all of them."""
     # We deal with a lot of details here, but rather one by one, and split makes
-    # no sense, pylint: disable=too-many-branches,too-many-locals,too-many-statements
+    # no sense, pylint: disable=too-many-branches
 
     assert main_module.isCompiledPythonModule()
 
-    # The global context used to generate code.
-    global_context = CodeGeneration.makeGlobalContext()
-
     # assert main_module in ModuleRegistry.getDoneModules()
 
     # We might have chosen to include it as bytecode, and only compiled it for
@@ -324,11 +291,11 @@ def makeSourceDirectory(main_module):
     # drop the frozen one.
     # TODO: This really should be done when the compiled module comes into
     # existence.
-    for module in ModuleRegistry.getDoneUserModules():
+    for module in ModuleRegistry.getDoneModules():
         if module.isCompiledPythonModule():
             uncompiled_module = ModuleRegistry.getUncompiledModule(
-                module_name     = module.getFullName(),
-                module_filename = module.getCompileTimeFilename()
+                module_name=module.getFullName(),
+                module_filename=module.getCompileTimeFilename(),
             )
 
             if uncompiled_module is not None:
@@ -336,8 +303,10 @@ def makeSourceDirectory(main_module):
                 # module. Some uncompiled modules may have been asked by the user
                 # or technically required. By default, frozen code if it exists
                 # is preferred, as it will be from standalone mode adding it.
-                if uncompiled_module.isUserProvided() or \
-                   uncompiled_module.isTechnical():
+                if (
+                    uncompiled_module.isUserProvided()
+                    or uncompiled_module.isTechnical()
+                ):
                     ModuleRegistry.removeDoneModule(module)
                 else:
                     ModuleRegistry.removeUncompiledModule(uncompiled_module)
@@ -345,17 +314,14 @@ def makeSourceDirectory(main_module):
     # Lets check if the recurse-to modules are actually present, and warn the
     # user if one of those was not found.
     for any_case_module in Options.getShallFollowModules():
-        if '*' in any_case_module or '{' in any_case_module:
+        if "*" in any_case_module or "{" in any_case_module:
             continue
 
-        for module in ModuleRegistry.getDoneUserModules():
+        for module in ModuleRegistry.getDoneModules():
             if module.getFullName() == any_case_module:
                 break
         else:
-            warning(
-                "Didn't recurse to '%s', apparently not used." % \
-                any_case_module
-            )
+            general.warning("Not recursing to unused '%s'." % any_case_module)
 
     # Prepare code generation, i.e. execute finalization for it.
     for module in ModuleRegistry.getDoneModules():
@@ -363,113 +329,77 @@ def makeSourceDirectory(main_module):
             Finalization.prepareCodeGeneration(module)
 
     # Pick filenames.
-    source_dir = getSourceDirectoryPath(main_module)
+    source_dir = OutputDirectories.getSourceDirectoryPath()
 
     module_filenames = pickSourceFilenames(
-        source_dir = source_dir,
-        modules    = ModuleRegistry.getDoneModules()
+        source_dir=source_dir, modules=ModuleRegistry.getDoneModules()
     )
 
-    # First pass, generate code and use constants doing so, but prepare the
-    # final code generation only, because constants code will be added at the
-    # end only.
-    prepared_modules = {}
-
+    # Generate code for modules.
     for module in ModuleRegistry.getDoneModules():
         if module.isCompiledPythonModule():
             c_filename = module_filenames[module]
 
-            prepared_modules[c_filename] = CodeGeneration.prepareModuleCode(
-                global_context = global_context,
-                module         = module,
-                module_name    = module.getFullName(),
-            )
-
-            # Main code constants need to be allocated already too.
-            if module is main_module and not Options.shallMakeModule():
-                prepared_modules[c_filename][1].getConstantCode(0)
-
-    # Second pass, generate the actual module code into the files.
-    for module in ModuleRegistry.getDoneModules():
-        if module.isCompiledPythonModule():
-            c_filename = module_filenames[module]
-
-            template_values, module_context = prepared_modules[c_filename]
-
             source_code = CodeGeneration.generateModuleCode(
-                module_context  = module_context,
-                template_values = template_values
+                module=module,
+                data_filename=os.path.basename(c_filename + "onst"),  # Really .const
             )
 
-            writeSourceCode(
-                filename    = c_filename,
-                source_code = source_code
-            )
+            writeSourceCode(filename=c_filename, source_code=source_code)
 
             if Options.isShowInclusion():
-                info("Included compiled module '%s'." % module.getFullName())
+                inclusion_logger.info(
+                    "Included compiled module '%s'." % module.getFullName()
+                )
         elif module.isPythonShlibModule():
-            target_filename = os.path.join(
-                getStandaloneDirectoryPath(main_module),
-                *module.getFullName().split('.')
-            )
+            addShlibEntryPoint(module)
 
-            if Utils.getOS() == "Windows":
-                target_filename += ".pyd"
-            else:
-                target_filename += ".so"
-
-            target_dir = os.path.dirname(target_filename)
-
-            if not os.path.isdir(target_dir):
-                makePath(target_dir)
-
-            shutil.copyfile(
-                module.getFilename(),
-                target_filename
-            )
-
-            standalone_entry_points.append(
-                (
-                    module.getFilename(),
-                    target_filename,
-                    module.getPackage()
+            if Options.isShowInclusion():
+                inclusion_logger.info(
+                    "Included extension module '%s'." % module.getFullName()
                 )
-            )
         elif module.isUncompiledPythonModule():
-            pass
+            if Options.isShowInclusion():
+                inclusion_logger.info(
+                    "Included uncompiled module '%s'." % module.getFullName()
+                )
         else:
             assert False, module
 
+    (
+        helper_decl_code,
+        helper_impl_code,
+        constants_header_code,
+        constants_body_code,
+    ) = CodeGeneration.generateHelpersCode()
+
     writeSourceCode(
-        filename    = os.path.join(
-            source_dir,
-            "__constants.c"
-        ),
-        source_code = ConstantCodes.getConstantsDefinitionCode(
-            context = global_context
+        filename=os.path.join(source_dir, "__helpers.h"), source_code=helper_decl_code
+    )
+
+    writeSourceCode(
+        filename=os.path.join(source_dir, "__helpers.c"), source_code=helper_impl_code
+    )
+
+    writeSourceCode(
+        filename=os.path.join(source_dir, "__constants.h"),
+        source_code=constants_header_code,
+    )
+
+    writeSourceCode(
+        filename=os.path.join(source_dir, "__constants.c"),
+        source_code=constants_body_code,
+    )
+
+    for filename, source_code in Plugins.getExtraCodeFiles().items():
+        target_dir = os.path.join(source_dir, "plugins")
+
+        if not os.path.isdir(target_dir):
+            makePath(target_dir)
+
+        writeSourceCode(
+            filename=os.path.join(target_dir, filename), source_code=source_code
         )
-    )
-
-    helper_decl_code, helper_impl_code = CodeGeneration.generateHelpersCode(
-        ModuleRegistry.getDoneUserModules()
-    )
-
-    writeSourceCode(
-        filename    = os.path.join(
-            source_dir,
-            "__helpers.h"
-        ),
-        source_code = helper_decl_code
-    )
-
-    writeSourceCode(
-        filename    = os.path.join(
-            source_dir,
-            "__helpers.c"
-        ),
-        source_code = helper_impl_code
-    )
 
 
 def _asBoolStr(value):
@@ -482,32 +412,33 @@ def runScons(main_module, quiet):
     # pylint: disable=too-many-branches,too-many-statements
 
     options = {
-        "name"            : os.path.basename(
-            getTreeFilenameWithSuffix(main_module, "")
+        "name": os.path.basename(
+            OutputDirectories.getTreeFilenameWithSuffix(main_module, "")
+        ),
+        "result_name": OutputDirectories.getResultBasepath(),
+        "source_dir": OutputDirectories.getSourceDirectoryPath(),
+        "debug_mode": _asBoolStr(Options.isDebug()),
+        "python_debug": _asBoolStr(Options.isPythonDebug()),
+        "unstripped_mode": _asBoolStr(Options.isUnstripped()),
+        "module_mode": _asBoolStr(Options.shallMakeModule()),
+        "full_compat": _asBoolStr(Options.isFullCompat()),
+        "experimental": ",".join(Options.getExperimentalIndications()),
+        "trace_mode": _asBoolStr(Options.shallTraceExecution()),
+        "python_version": python_version_str,
+        "target_arch": Utils.getArchitecture(),
+        "python_prefix": sys.prefix,
+        "nuitka_src": SconsInterface.getSconsDataPath(),
+        "nuitka_cache": getCacheDir(),
+        "module_count": "%d"
+        % (
+            1
+            + len(ModuleRegistry.getDoneModules())
+            + len(ModuleRegistry.getUncompiledNonTechnicalModules())
         ),
-        "result_name"     : getResultBasepath(main_module),
-        "source_dir"      : getSourceDirectoryPath(main_module),
-        "debug_mode"      : _asBoolStr(Options.isDebug()),
-        "python_debug"    : _asBoolStr(Options.isPythonDebug()),
-        "unstripped_mode" : _asBoolStr(Options.isUnstripped()),
-        "module_mode"     : _asBoolStr(Options.shallMakeModule()),
-        "full_compat"     : _asBoolStr(Options.isFullCompat()),
-        "experimental"    : ','.join(Options.getExperimentalIndications()),
-        "trace_mode"      : _asBoolStr(Options.shallTraceExecution()),
-        "python_version"  : python_version_str,
-        "target_arch"     : Utils.getArchitecture(),
-        "python_prefix"   : sys.prefix,
-        "nuitka_src"      : SconsInterface.getSconsDataPath(),
-        "nuitka_cache"    : getCacheDir(),
-        "module_count"    : "%d" % (
-            1 + \
-            len(ModuleRegistry.getDoneUserModules()) + \
-            len(ModuleRegistry.getUncompiledNonTechnicalModules())
-        )
     }
 
     if not Options.shallMakeModule():
-        options["result_exe"] = getResultFullpath(main_module)
+        options["result_exe"] = OutputDirectories.getResultFullpath()
 
     # Ask Scons to cache on Windows, except where the directory is thrown
     # away. On non-Windows you can should use ccache instead.
@@ -517,9 +448,7 @@ def runScons(main_module, quiet):
     if Options.isLto():
         options["lto_mode"] = "true"
 
-    # For AnaConda default to trying static lib python library, which
-    # normally is just not available or if it is even unusable.
-    if "Anaconda" in sys.version:
+    if Options.shallUseStaticLibPython():
         options["static_libpython"] = "true"
 
     if Options.shallDisableConsoleWindow():
@@ -528,9 +457,7 @@ def runScons(main_module, quiet):
     if Options.isStandaloneMode():
         options["standalone_mode"] = "true"
 
-    if not Options.isStandaloneMode() and \
-       not Options.shallMakeModule() and \
-       isUninstalledPython():
+    if Options.shallTreatUninstalledPython():
         options["uninstalled_python"] = "true"
 
     if ModuleRegistry.getUncompiledTechnicalModules():
@@ -548,11 +475,14 @@ def runScons(main_module, quiet):
         msvc_version = Options.getMsvcVersion()
 
         msvc_version = msvc_version.replace("exp", "Exp")
-        if '.' not in msvc_version:
+        if "." not in msvc_version:
             msvc_version += ".0"
 
         options["msvc_version"] = msvc_version
 
+    if Utils.getOS() == "Windows":
+        options["noelf_mode"] = "true"
+
     if Options.isClang():
         options["clang_mode"] = "true"
 
@@ -583,6 +513,9 @@ def runScons(main_module, quiet):
     if "trace_imports" in Options.getPythonFlags():
         options["python_sysflag_verbose"] = "true"
 
+    if "no_randomization" in Options.getPythonFlags():
+        options["python_sysflag_no_randomization"] = "true"
+
     if python_version < 300 and sys.flags.unicode:
         options["python_sysflag_unicode"] = "true"
 
@@ -593,6 +526,23 @@ def runScons(main_module, quiet):
     if abiflags:
         options["abiflags"] = abiflags
 
+    cpp_defines = Plugins.getPreprocessorSymbols()
+    if cpp_defines:
+        options["cpp_defines"] = ",".join(
+            "%s%s%s" % (key, "=" if value else "", value or "")
+            for key, value in cpp_defines.items()
+        )
+
+    link_libraries = Plugins.getExtraLinkLibraries()
+    if link_libraries:
+        options["link_libraries"] = ",".join(link_libraries)
+
+    if Options.shallMakeModule():
+        options["module_suffix"] = getSharedLibrarySuffix(preferred=True)
+
+    if Options.shallRunInDebugger():
+        options["full_names"] = "true"
+
     return SconsInterface.runScons(options, quiet), options
 
 
@@ -605,7 +555,7 @@ def writeSourceCode(filename, source_code):
         with open(filename, "wb") as output_file:
             output_file.write(source_code.encode("latin1"))
     else:
-        with open(filename, 'w') as output_file:
+        with open(filename, "w") as output_file:
             output_file.write(source_code)
 
 
@@ -628,7 +578,7 @@ def callExecPython(args, clean_path, add_path):
 
     if add_path:
         if "PYTHONPATH" in os.environ:
-            os.environ["PYTHONPATH"] += ':' + Options.getOutputDir()
+            os.environ["PYTHONPATH"] += ":" + Options.getOutputDir()
         else:
             os.environ["PYTHONPATH"] = Options.getOutputDir()
 
@@ -643,77 +593,62 @@ def callExecPython(args, clean_path, add_path):
 
 
 def executeMain(binary_filename, clean_path):
-    args = (binary_filename, binary_filename)
-
     if Options.shallRunInDebugger():
-        gdb_path = Execution.getExecutablePath("gdb")
+        args = Execution.wrapCommandForDebuggerForExec(binary_filename)
+    else:
+        args = (binary_filename, binary_filename)
 
-        if gdb_path is None:
-            sys.exit("Error, no 'gdb' binary found in path.")
-
-        args = (gdb_path, "gdb", "-ex=run", "-ex=where", "--args", binary_filename)
-
-    callExecPython(
-        clean_path = clean_path,
-        add_path   = False,
-        args       = args
-    )
+    callExecPython(clean_path=clean_path, add_path=False, args=args)
 
 
 def executeModule(tree, clean_path):
     python_command = "__import__('%s')" % tree.getName()
 
-    args = (
-        sys.executable,
-        "python",
-        "-c",
-        python_command,
-    )
+    if Options.shallRunInDebugger():
+        args = Execution.wrapCommandForDebuggerForExec(
+            sys.executable, "-c", python_command
+        )
+    else:
+        args = (sys.executable, "python", "-c", python_command)
 
-    callExecPython(
-        clean_path = clean_path,
-        add_path   = True,
-        args       = args
-    )
+    callExecPython(clean_path=clean_path, add_path=True, args=args)
 
 
 def compileTree(main_module):
-    source_dir = getSourceDirectoryPath(main_module)
+    source_dir = OutputDirectories.getSourceDirectoryPath()
 
     if not Options.shallOnlyExecCCompilerCall():
-        # Now build the target language code for the whole tree.
-        makeSourceDirectory(
-            main_module = main_module
-        )
-
-        frozen_code = generateBytecodeFrozenCode()
-
-        if frozen_code is not None:
-            writeSourceCode(
-                filename    = os.path.join(
-                    source_dir,
-                    "__frozen.c"
-                ),
-                source_code = frozen_code
+        if Options.isShowProgress() or Options.isShowMemory():
+            general.info(
+                "Total memory usage before generating C code: {memory}:".format(
+                    memory=MemoryUsage.getHumanReadableProcessMemoryUsage()
+                )
             )
+        # Now build the target language code for the whole tree.
+        makeSourceDirectory(main_module=main_module)
 
-        writeBinaryData(
-            filename    = os.path.join(
-                source_dir,
-                "__constants.bin"
-            ),
-            binary_data = ConstantCodes.stream_data.getBytes()
+        bytecode_accessor = ConstantAccessor(
+            data_filename="__bytecode.const", top_level_name="bytecode_data"
         )
+
+        # This should take all bytecode values, even ones needed for frozen or
+        # not produce anything.
+        loader_code = LoaderCodes.getMetapathLoaderBodyCode(bytecode_accessor)
+
+        writeSourceCode(
+            filename=os.path.join(source_dir, "__loader.c"), source_code=loader_code
+        )
+
     else:
-        source_dir = getSourceDirectoryPath(main_module)
+        source_dir = OutputDirectories.getSourceDirectoryPath()
 
         if not os.path.isfile(os.path.join(source_dir, "__helpers.h")):
             sys.exit("Error, no previous build directory exists.")
 
     if Options.isShowProgress() or Options.isShowMemory():
-        info(
+        general.info(
             "Total memory usage before running scons: {memory}:".format(
-                memory = MemoryUsage.getHumanReadableProcessMemoryUsage()
+                memory=MemoryUsage.getHumanReadableProcessMemoryUsage()
             )
         )
 
@@ -726,11 +661,12 @@ def compileTree(main_module):
     if Options.shallNotDoExecCCompilerCall():
         return True, {}
 
+    # TODO: On Windows, we could run this in parallel to Scons, on Linux we need it
+    # for linking.
+    runDataComposer(source_dir)
+
     # Run the Scons to build things.
-    result, options = runScons(
-        main_module = main_module,
-        quiet       = not Options.isShowScons()
-    )
+    result, options = runScons(main_module=main_module, quiet=not Options.isShowScons())
 
     return result, options
 
@@ -753,22 +689,23 @@ Nuitka is very syntax compatible with standard Python. It is currently running
 with Python version '%s', you might want to specify more clearly with the use
 of the precise Python interpreter binary and '-m nuitka', e.g. use this
 'python%s -m nuitka' option, if that's not the one the program expects.
-""" % (python_version_str, suggested_python_version_str)
+""" % (
+            python_version_str,
+            suggested_python_version_str,
+        )
 
     sys.exit(error_message)
 
 
-data_files = []
-
 def main():
-    """ Main program flow of Nuitka
+    """Main program flow of Nuitka
 
-        At this point, options will be parsed already, Nuitka will be executing
-        in the desired version of Python with desired flags, and we just get
-        to execute the task assigned.
+    At this point, options will be parsed already, Nuitka will be executing
+    in the desired version of Python with desired flags, and we just get
+    to execute the task assigned.
 
-        We might be asked to only re-compile generated C, dump only an XML
-        representation of the internal node tree after optimization, etc.
+    We might be asked to only re-compile generated C, dump only an XML
+    representation of the internal node tree after optimization, etc.
     """
 
     # Main has to fulfill many options, leading to many branches and statements
@@ -778,31 +715,12 @@ def main():
     # Inform the importing layer about the main script directory, so it can use
     # it when attempting to follow imports.
     Importing.setMainScriptDirectory(
-        main_dir = os.path.dirname(os.path.abspath(filename))
+        main_dir=os.path.dirname(os.path.abspath(filename))
     )
 
-    # Detect to be frozen modules if any, so we can consider to not recurse
-    # to them.
-    if Options.isStandaloneMode():
-        for module in detectEarlyImports():
-            ModuleRegistry.addUncompiledModule(module)
-
-            if module.getName() == "site":
-                origin_prefix_filename = os.path.join(
-                    os.path.dirname(module.getCompileTimeFilename()),
-                    "orig-prefix.txt"
-                )
-
-                if os.path.isfile(origin_prefix_filename):
-                    data_files.append(
-                        (filename, "orig-prefix.txt")
-                    )
-
     # Turn that source code into a node tree structure.
     try:
-        main_module = createNodeTree(
-            filename = filename
-        )
+        main_module = createNodeTree(filename=filename)
     except (SyntaxError, IndentationError) as e:
         handleSyntaxError(e)
 
@@ -812,9 +730,7 @@ def main():
             dumpTreeXML(module)
     else:
         # Make the actual compilation.
-        result, options = compileTree(
-            main_module = main_module
-        )
+        result, options = compileTree(main_module=main_module)
 
         # Exit if compilation failed.
         if not result:
@@ -826,72 +742,38 @@ def main():
 
             sys.exit(0)
 
+        executePostProcessing()
+
         if Options.isStandaloneMode():
             binary_filename = options["result_exe"]
 
-            standalone_entry_points.insert(
-                0,
-                (binary_filename, binary_filename, None)
-            )
+            setMainEntryPoint(binary_filename)
 
-            dist_dir = getStandaloneDirectoryPath(main_module)
-
-            for module in ModuleRegistry.getDoneUserModules():
-                standalone_entry_points.extend(
-                    Plugins.considerExtraDlls(dist_dir, module)
-                )
-
-            for module in ModuleRegistry.getUncompiledModules():
-                standalone_entry_points.extend(
-                    Plugins.considerExtraDlls(dist_dir, module)
-                )
-
-            copyUsedDLLs(
-                source_dir              = getSourceDirectoryPath(main_module),
-                dist_dir                = dist_dir,
-                standalone_entry_points = standalone_entry_points
-            )
+            dist_dir = OutputDirectories.getStandaloneDirectoryPath()
 
             for module in ModuleRegistry.getDoneModules():
-                data_files.extend(
-                    Plugins.considerDataFiles(module)
-                )
+                addIncludedEntryPoints(Plugins.considerExtraDlls(dist_dir, module))
 
-            for source_filename, target_filename in data_files:
-                target_filename = os.path.join(
-                    getStandaloneDirectoryPath(main_module),
-                    target_filename
-                )
+            copyUsedDLLs(
+                source_dir=OutputDirectories.getSourceDirectoryPath(),
+                dist_dir=dist_dir,
+                standalone_entry_points=getStandardEntryPoints(),
+            )
 
-                makePath(os.path.dirname(target_filename))
+            copyDataFiles(dist_dir=dist_dir)
 
-                shutil.copy2(
-                    source_filename,
-                    target_filename
-                )
+            Plugins.onStandaloneDistributionFinished(dist_dir)
 
         # Remove the source directory (now build directory too) if asked to.
         if Options.isRemoveBuildDir():
             removeDirectory(
-                path          = getSourceDirectoryPath(main_module),
-                ignore_errors = False
-            )
-
-        # Modules should not be executable, but Scons creates them like it, fix
-        # it up here. TODO: Move inside scons file and avoid subprocess call.
-        if Utils.getOS() != "Windows" and Options.shallMakeModule():
-            subprocess.call(
-                (
-                    "chmod",
-                    "-x",
-                    getResultFullpath(main_module)
-                )
+                path=OutputDirectories.getSourceDirectoryPath(), ignore_errors=False
             )
 
         if Options.shallMakeModule() and Options.shallCreatePyiFile():
-            pyi_filename = getResultBasepath(main_module) + ".pyi"
+            pyi_filename = OutputDirectories.getResultBasepath() + ".pyi"
 
-            with open(pyi_filename, 'w') as pyi_file:
+            with open(pyi_filename, "w") as pyi_file:
                 pyi_file.write(
                     """\
 # This file was generated by Nuitka and describes the types of the
@@ -912,26 +794,24 @@ def main():
 # now. This was decided by PEP 484 designers.
 __name__ = ...
 
-""" % {
-                        "imports" : '\n'.join(
+"""
+                    % {
+                        "imports": "\n".join(
                             "import %s" % module_name
-                            for module_name in
-                            getImportedNames()
+                            for module_name in getImportedNames()
                         )
-
                     }
                 )
 
-
         # Execute the module immediately if option was given.
         if Options.shallExecuteImmediately():
             if Options.shallMakeModule():
                 executeModule(
-                    tree       = main_module,
-                    clean_path = Options.shallClearPythonPathEnvironment()
+                    tree=main_module,
+                    clean_path=Options.shallClearPythonPathEnvironment(),
                 )
             else:
                 executeMain(
-                    binary_filename = getResultFullpath(main_module),
-                    clean_path      = Options.shallClearPythonPathEnvironment()
+                    binary_filename=OutputDirectories.getResultFullpath(),
+                    clean_path=Options.shallClearPythonPathEnvironment(),
                 )
