@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -20,29 +20,22 @@
 """
 
 template_publish_exception_to_handler = """\
-if ( %(keeper_tb)s == NULL )
-{
+if (%(keeper_tb)s == NULL) {
     %(keeper_tb)s = %(tb_making)s;
-}
-else if ( %(keeper_lineno)s != 0 )
-{
-    %(keeper_tb)s = ADD_TRACEBACK( %(keeper_tb)s, %(frame_identifier)s, %(keeper_lineno)s );
+} else if (%(keeper_lineno)s != 0) {
+    %(keeper_tb)s = ADD_TRACEBACK(%(keeper_tb)s, %(frame_identifier)s, %(keeper_lineno)s);
 }
 """
 
 template_error_catch_quick_exception = """\
-if ( %(condition)s )
-{
-    if ( !ERROR_OCCURRED() )
-    {
-        exception_type = %(quick_exception)s;
-        Py_INCREF( exception_type );
-        exception_value = NULL;
-        exception_tb = NULL;
-    }
-    else
-    {
-        FETCH_ERROR_OCCURRED( &exception_type, &exception_value, &exception_tb );
+if (%(condition)s) {
+    if (!ERROR_OCCURRED()) {
+        %(exception_type)s = %(quick_exception)s;
+        Py_INCREF(%(exception_type)s);
+        %(exception_value)s = NULL;
+        %(exception_tb)s = NULL;
+    } else {
+        FETCH_ERROR_OCCURRED(&%(exception_type)s, &%(exception_value)s, &%(exception_tb)s);
     }
 %(release_temps)s
 
@@ -52,11 +45,10 @@ if ( %(condition)s )
 }"""
 
 template_error_catch_exception = """\
-if ( %(condition)s )
-{
-    assert( ERROR_OCCURRED() );
+if (%(condition)s) {
+    assert(ERROR_OCCURRED());
 
-    FETCH_ERROR_OCCURRED( &exception_type, &exception_value, &exception_tb );
+    FETCH_ERROR_OCCURRED(&%(exception_type)s, &%(exception_value)s, &%(exception_tb)s);
 %(release_temps)s
 
 %(line_number_code)s
@@ -65,8 +57,7 @@ if ( %(condition)s )
 }"""
 
 template_error_format_string_exception = """\
-if ( %(condition)s )
-{
+if (%(condition)s) {
 %(release_temps)s
 %(set_exception)s
 
@@ -76,5 +67,18 @@ if ( %(condition)s )
 }
 """
 
-from . import TemplateDebugWrapper # isort:skip
+template_error_format_name_error_exception = """\
+if (unlikely(%(condition)s)) {
+%(release_temps)s
+%(set_exception)s
+
+%(line_number_code)s
+%(var_description_code)s
+    goto %(exception_exit)s;
+}
+"""
+
+
+from . import TemplateDebugWrapper  # isort:skip
+
 TemplateDebugWrapper.checkDebug(globals())
