@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,22 +25,22 @@ and then later active again, via another import, and in this case, we should
 not start anew, but reuse what we already found out about it.
 """
 
+import os
+
 from nuitka.plugins.Plugins import Plugins
-from nuitka.utils import Utils
+from nuitka.utils.FileOperations import relpath
 
 imported_modules = {}
 imported_by_name = {}
 
+
 def addImportedModule(imported_module):
-    module_filename = Utils.relpath(imported_module.getFilename())
+    module_filename = relpath(imported_module.getFilename())
 
-    if Utils.basename(module_filename) == "__init__.py":
-        module_filename = Utils.dirname(module_filename)
+    if os.path.basename(module_filename) == "__init__.py":
+        module_filename = os.path.dirname(module_filename)
 
-    key = (
-        module_filename,
-        imported_module.getFullName()
-    )
+    key = (module_filename, imported_module.getFullName())
 
     if key in imported_modules:
         assert imported_module is imported_modules[key], key
@@ -70,9 +70,28 @@ def getImportedModuleByName(full_name):
     return imported_by_name[full_name]
 
 
-def getImportedModuleByPath(module_relpath):
+def getImportedModuleByNameAndPath(full_name, module_relpath):
+    if module_relpath is None:
+        # pyi deps only
+        return getImportedModuleByName(full_name)
+
+    if os.path.basename(module_relpath) == "__init__.py":
+        module_relpath = os.path.dirname(module_relpath)
+
+    return imported_modules[module_relpath, full_name]
+
+
+def getImportedModuleByPath(module_relpath, module_package):
     for key in imported_modules:
         if key[0] == module_relpath:
+            module = imported_modules[key]
+
+            if (
+                module.getCompileTimeFilename().endswith("__init__.py")
+                and module.getFullName().getPackageName() != module_package
+            ):
+                continue
+
             return imported_modules[key]
 
     raise KeyError(module_relpath)
