@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -28,25 +28,27 @@ import math
 from nuitka.PythonVersions import python_version
 from nuitka.specs import BuiltinParameterSpecs
 
-from .ExpressionBases import ExpressionChildrenHavingBase
+from .ExpressionBases import (
+    ExpressionChildHavingBase,
+    ExpressionChildrenHavingBase,
+)
+from .IterationHandles import (
+    ConstantIterationHandleRange1,
+    ConstantIterationHandleRange2,
+    ConstantIterationHandleRange3,
+)
 from .NodeMakingHelpers import makeConstantReplacementNode
-from .shapes.BuiltinTypeShapes import ShapeTypeList, ShapeTypeXrange
+from .shapes.BuiltinTypeShapes import tshape_list, tshape_xrange
 
 
-class ExpressionBuiltinRangeBase(ExpressionChildrenHavingBase):
-    """ Base class for range nodes with 1/2/3 arguments. """
+class ExpressionBuiltinRangeMixin(object):
+    """ Mixin class for range nodes with 1/2/3 arguments. """
 
     builtin_spec = BuiltinParameterSpecs.builtin_range_spec
 
-    def __init__(self, values, source_ref):
-        ExpressionChildrenHavingBase.__init__(
-            self,
-            values     = values,
-            source_ref = source_ref
-        )
-
-    def getTypeShape(self):
-        return ShapeTypeList
+    @staticmethod
+    def getTypeShape():
+        return tshape_list
 
     def getTruthValue(self):
         length = self.getIterationLength()
@@ -64,9 +66,11 @@ class ExpressionBuiltinRangeBase(ExpressionChildrenHavingBase):
             if child.getIntegerValue() is None:
                 return True
 
-            if python_version >= 270 and \
-               child.isExpressionConstantRef() and \
-               type(child.getConstant()) is float:
+            if (
+                python_version >= 270
+                and child.isExpressionConstantRef()
+                and type(child.getConstant()) is float
+            ):
                 return True
 
         return False
@@ -80,9 +84,11 @@ class ExpressionBuiltinRangeBase(ExpressionChildrenHavingBase):
             if child.getIntegerValue() is None:
                 return True
 
-            if python_version >= 270 and \
-               child.isExpressionConstantRef() and \
-               type(child.getConstant()) is float:
+            if (
+                python_version >= 270
+                and child.isExpressionConstantRef()
+                and type(child.getConstant()) is float
+            ):
                 return True
 
         step = self.getStep()
@@ -104,13 +110,10 @@ class ExpressionBuiltinRangeBase(ExpressionChildrenHavingBase):
             return self, None, None
 
         return trace_collection.getCompileTimeComputationResult(
-            node        = self,
-            computation = lambda : self.builtin_spec.simulateCall(
-                given_values
-            ),
-            description = "Built-in call to '%s' computed." % (
-                self.builtin_spec.getName()
-            )
+            node=self,
+            computation=lambda: self.builtin_spec.simulateCall(given_values),
+            description="Built-in call to '%s' computed."
+            % (self.builtin_spec.getName()),
         )
 
     def computeExpressionIter1(self, iter_node, trace_collection):
@@ -120,10 +123,10 @@ class ExpressionBuiltinRangeBase(ExpressionChildrenHavingBase):
 
         if iteration_length is not None and iteration_length > 256:
             result = makeExpressionBuiltinXrange(
-                low        = self.getLow(),
-                high       = self.getHigh(),
-                step       = self.getStep(),
-                source_ref = self.getSourceReference()
+                low=self.getLow(),
+                high=self.getHigh(),
+                step=self.getStep(),
+                source_ref=self.getSourceReference(),
             )
 
             self.parent.replaceChild(self, result)
@@ -132,7 +135,7 @@ class ExpressionBuiltinRangeBase(ExpressionChildrenHavingBase):
             return (
                 iter_node,
                 "new_expression",
-                "Replaced 'range' with 'xrange' built-in call for iteration."
+                "Replaced 'range' with 'xrange' built-in call for iteration.",
             )
 
         # No exception will be raised on ranges.
@@ -155,36 +158,23 @@ class ExpressionBuiltinRangeBase(ExpressionChildrenHavingBase):
         return None
 
 
-class ExpressionBuiltinRange1(ExpressionBuiltinRangeBase):
+class ExpressionBuiltinRange1(ExpressionBuiltinRangeMixin, ExpressionChildHavingBase):
     kind = "EXPRESSION_BUILTIN_RANGE1"
 
-    named_children = (
-        "low",
-    )
+    named_child = "low"
+    getLow = ExpressionChildrenHavingBase.childGetter("low")
 
     def __init__(self, low, source_ref):
         assert low is not None
-
-        ExpressionBuiltinRangeBase.__init__(
-            self,
-            values     = {
-                "low" : low,
-            },
-            source_ref = source_ref
-        )
-
-    getLow = ExpressionChildrenHavingBase.childGetter("low")
-
-    def computeExpression(self, trace_collection):
         assert python_version < 300
 
-        low  = self.getLow()
+        ExpressionChildHavingBase.__init__(self, value=low, source_ref=source_ref)
+
+    def computeExpression(self, trace_collection):
+        low = self.getLow()
 
         return self.computeBuiltinSpec(
-            trace_collection = trace_collection,
-            given_values     = (
-                low,
-            )
+            trace_collection=trace_collection, given_values=(low,)
         )
 
     def getIterationLength(self):
@@ -195,6 +185,13 @@ class ExpressionBuiltinRange1(ExpressionBuiltinRangeBase):
 
         return max(0, low)
 
+    def getIterationHandle(self):
+        low = self.getLow().getIntegerValue()
+        if low is None:
+            return None
+
+        return ConstantIterationHandleRange1(low, self.source_ref)
+
     def getIterationValue(self, element_index):
         length = self.getIterationLength()
 
@@ -206,51 +203,40 @@ class ExpressionBuiltinRange1(ExpressionBuiltinRangeBase):
 
         # TODO: Make sure to cast element_index to what CPython will give, for
         # now a downcast will do.
-        return makeConstantReplacementNode(
-            constant = int(element_index),
-            node     = self
-        )
+        return makeConstantReplacementNode(constant=int(element_index), node=self)
 
     def isKnownToBeIterable(self, count):
         return count is None or count == self.getIterationLength()
 
 
-class ExpressionBuiltinRange2(ExpressionBuiltinRangeBase):
+class ExpressionBuiltinRange2(
+    ExpressionBuiltinRangeMixin, ExpressionChildrenHavingBase
+):
     kind = "EXPRESSION_BUILTIN_RANGE2"
 
     named_children = ("low", "high")
+    getLow = ExpressionChildrenHavingBase.childGetter("low")
+    getHigh = ExpressionChildrenHavingBase.childGetter("high")
 
     def __init__(self, low, high, source_ref):
-        ExpressionBuiltinRangeBase.__init__(
-            self,
-            values     = {
-                "low"  : low,
-                "high" : high
-            },
-            source_ref = source_ref
+        ExpressionChildrenHavingBase.__init__(
+            self, values={"low": low, "high": high}, source_ref=source_ref
         )
 
-    getLow  = ExpressionChildrenHavingBase.childGetter("low")
-    getHigh = ExpressionChildrenHavingBase.childGetter("high")
-
     builtin_spec = BuiltinParameterSpecs.builtin_range_spec
 
     def computeExpression(self, trace_collection):
         assert python_version < 300
 
-        low  = self.getLow()
+        low = self.getLow()
         high = self.getHigh()
 
         return self.computeBuiltinSpec(
-            trace_collection = trace_collection,
-            given_values     = (
-                low,
-                high
-            )
+            trace_collection=trace_collection, given_values=(low, high)
         )
 
     def getIterationLength(self):
-        low  = self.getLow()
+        low = self.getLow()
         high = self.getHigh()
 
         low = low.getIntegerValue()
@@ -265,8 +251,19 @@ class ExpressionBuiltinRange2(ExpressionBuiltinRangeBase):
 
         return max(0, high - low)
 
+    def getIterationHandle(self):
+        low = self.getLow().getIntegerValue()
+        if low is None:
+            return None
+
+        high = self.getHigh().getIntegerValue()
+        if high is None:
+            return None
+
+        return ConstantIterationHandleRange2(low, high, self.source_ref)
+
     def getIterationValue(self, element_index):
-        low  = self.getLow()
+        low = self.getLow()
         high = self.getHigh()
 
         low = low.getIntegerValue()
@@ -284,59 +281,42 @@ class ExpressionBuiltinRange2(ExpressionBuiltinRangeBase):
         if result >= high:
             return None
         else:
-            return makeConstantReplacementNode(
-                constant = result,
-                node     = self
-            )
+            return makeConstantReplacementNode(constant=result, node=self)
 
     def isKnownToBeIterable(self, count):
         return count is None or count == self.getIterationLength()
 
 
-class ExpressionBuiltinRange3(ExpressionBuiltinRangeBase):
+class ExpressionBuiltinRange3(
+    ExpressionBuiltinRangeMixin, ExpressionChildrenHavingBase
+):
     kind = "EXPRESSION_BUILTIN_RANGE3"
 
-    named_children = (
-        "low",
-        "high",
-        "step"
-    )
-
-    def __init__(self, low, high, step, source_ref):
-        ExpressionBuiltinRangeBase.__init__(
-            self,
-            values     = {
-                "low"  : low,
-                "high" : high,
-                "step" : step
-            },
-            source_ref = source_ref
-        )
-
-    getLow  = ExpressionChildrenHavingBase.childGetter("low")
+    named_children = ("low", "high", "step")
+    getLow = ExpressionChildrenHavingBase.childGetter("low")
     getHigh = ExpressionChildrenHavingBase.childGetter("high")
     getStep = ExpressionChildrenHavingBase.childGetter("step")
 
+    def __init__(self, low, high, step, source_ref):
+        ExpressionChildrenHavingBase.__init__(
+            self, values={"low": low, "high": high, "step": step}, source_ref=source_ref
+        )
+
     builtin_spec = BuiltinParameterSpecs.builtin_range_spec
 
     def computeExpression(self, trace_collection):
         assert python_version < 300
 
-        low  = self.getLow()
+        low = self.getLow()
         high = self.getHigh()
         step = self.getStep()
 
         return self.computeBuiltinSpec(
-            trace_collection = trace_collection,
-            given_values     = (
-                low,
-                high,
-                step
-            )
+            trace_collection=trace_collection, given_values=(low, high, step)
         )
 
     def getIterationLength(self):
-        low  = self.getLow()
+        low = self.getLow()
         high = self.getHigh()
         step = self.getStep()
 
@@ -379,8 +359,27 @@ class ExpressionBuiltinRange3(ExpressionBuiltinRangeBase):
     def canPredictIterationValues(self):
         return self.getIterationLength() is not None
 
+    def getIterationHandle(self):
+        low = self.getLow().getIntegerValue()
+        if low is None:
+            return None
+
+        high = self.getHigh().getIntegerValue()
+        if high is None:
+            return None
+
+        step = self.getStep().getIntegerValue()
+        if step is None:
+            return None
+
+        # Give up on this, will raise ValueError.
+        if step == 0:
+            return None
+
+        return ConstantIterationHandleRange3(low, high, step, self.source_ref)
+
     def getIterationValue(self, element_index):
-        low  = self.getLow().getIntegerValue()
+        low = self.getLow().getIntegerValue()
 
         if low is None:
             return None
@@ -397,29 +396,20 @@ class ExpressionBuiltinRange3(ExpressionBuiltinRangeBase):
         if result >= high:
             return None
         else:
-            return makeConstantReplacementNode(
-                constant = result,
-                node     = self
-            )
+            return makeConstantReplacementNode(constant=result, node=self)
 
     def isKnownToBeIterable(self, count):
         return count is None or count == self.getIterationLength()
 
 
-class ExpressionBuiltinXrangeBase(ExpressionChildrenHavingBase):
-    """ Base class for xrange nodes with 1/2/3 arguments. """
+class ExpressionBuiltinXrangeMixin(object):
+    """ Mixin class for xrange nodes with 1/2/3 arguments. """
 
     builtin_spec = BuiltinParameterSpecs.builtin_xrange_spec
 
-    def __init__(self, values, source_ref):
-        ExpressionChildrenHavingBase.__init__(
-            self,
-            values     = values,
-            source_ref = source_ref
-        )
-
-    def getTypeShape(self):
-        return ShapeTypeXrange
+    @staticmethod
+    def getTypeShape():
+        return tshape_xrange
 
     def canPredictIterationValues(self):
         return self.getIterationLength() is not None
@@ -470,17 +460,15 @@ class ExpressionBuiltinXrangeBase(ExpressionChildrenHavingBase):
             return self, None, None
 
         return trace_collection.getCompileTimeComputationResult(
-            node        = self,
-            computation = lambda : self.builtin_spec.simulateCall(
-                given_values
-            ),
-            description = "Built-in call to '%s' computed." % (
-                self.builtin_spec.getName()
-            )
+            node=self,
+            computation=lambda: self.builtin_spec.simulateCall(given_values),
+            description="Built-in call to '%s' computed."
+            % (self.builtin_spec.getName()),
         )
 
     def computeExpressionIter1(self, iter_node, trace_collection):
-        # No exception will be raised on xrange iteration.
+        # No exception will be raised on xrange iteration, but there is nothing to
+        # lower for, virtual method: pylint: disable=no-self-use
 
         return iter_node, None, None
 
@@ -497,34 +485,23 @@ class ExpressionBuiltinXrangeBase(ExpressionChildrenHavingBase):
         return None
 
 
-class ExpressionBuiltinXrange1(ExpressionBuiltinXrangeBase):
+class ExpressionBuiltinXrange1(ExpressionBuiltinXrangeMixin, ExpressionChildHavingBase):
     kind = "EXPRESSION_BUILTIN_XRANGE1"
 
-    named_children = (
-        "low",
-    )
+    named_child = "low"
+    getLow = ExpressionChildrenHavingBase.childGetter("low")
 
     def __init__(self, low, source_ref):
-        ExpressionBuiltinXrangeBase.__init__(
-            self,
-            values     = {
-                "low" : low,
-            },
-            source_ref = source_ref
-        )
+        ExpressionChildHavingBase.__init__(self, value=low, source_ref=source_ref)
 
     def computeExpression(self, trace_collection):
         low = self.getLow()
 
+        # TODO: Optimize this if self.getLow().getIntegerValue() is Not None
         return self.computeBuiltinSpec(
-            trace_collection = trace_collection,
-            given_values     = (
-                low,
-            )
+            trace_collection=trace_collection, given_values=(low,)
         )
 
-    getLow = ExpressionChildrenHavingBase.childGetter("low")
-
     def getIterationLength(self):
         low = self.getLow().getIntegerValue()
 
@@ -544,44 +521,33 @@ class ExpressionBuiltinXrange1(ExpressionBuiltinXrangeBase):
 
         # TODO: Make sure to cast element_index to what CPython will give, for
         # now a downcast will do.
-        return makeConstantReplacementNode(
-            constant = int(element_index),
-            node     = self
-        )
+        return makeConstantReplacementNode(constant=int(element_index), node=self)
 
 
-class ExpressionBuiltinXrange2(ExpressionBuiltinXrangeBase):
+class ExpressionBuiltinXrange2(
+    ExpressionBuiltinXrangeMixin, ExpressionChildrenHavingBase
+):
     kind = "EXPRESSION_BUILTIN_XRANGE2"
 
-    named_children = (
-        "low",
-        "high",
-    )
+    named_children = ("low", "high")
+    getLow = ExpressionChildrenHavingBase.childGetter("low")
+    getHigh = ExpressionChildrenHavingBase.childGetter("high")
 
     def __init__(self, low, high, source_ref):
-        ExpressionBuiltinXrangeBase.__init__(
-            self,
-            values     = {
-                "low"  : low,
-                "high" : high,
-            },
-            source_ref = source_ref
+        ExpressionChildrenHavingBase.__init__(
+            self, values={"low": low, "high": high}, source_ref=source_ref
         )
 
     def computeExpression(self, trace_collection):
-        low  = self.getLow()
+        low = self.getLow()
         high = self.getHigh()
 
         return self.computeBuiltinSpec(
-            trace_collection = trace_collection,
-            given_values     = (
-                low,
-                high
-            )
+            trace_collection=trace_collection, given_values=(low, high)
         )
 
     def getIterationLength(self):
-        low  = self.getLow()
+        low = self.getLow()
         high = self.getHigh()
 
         low = low.getIntegerValue()
@@ -597,7 +563,7 @@ class ExpressionBuiltinXrange2(ExpressionBuiltinXrangeBase):
         return max(0, high - low)
 
     def getIterationValue(self, element_index):
-        low  = self.getLow()
+        low = self.getLow()
         high = self.getHigh()
 
         low = low.getIntegerValue()
@@ -615,55 +581,35 @@ class ExpressionBuiltinXrange2(ExpressionBuiltinXrangeBase):
         if result >= high:
             return None
         else:
-            return makeConstantReplacementNode(
-                constant = result,
-                node     = self
-            )
-
-    getLow  = ExpressionChildrenHavingBase.childGetter("low")
-    getHigh = ExpressionChildrenHavingBase.childGetter("high")
+            return makeConstantReplacementNode(constant=result, node=self)
 
 
-class ExpressionBuiltinXrange3(ExpressionBuiltinXrangeBase):
+class ExpressionBuiltinXrange3(
+    ExpressionBuiltinXrangeMixin, ExpressionChildrenHavingBase
+):
     kind = "EXPRESSION_BUILTIN_XRANGE3"
 
-    named_children = (
-        "low",
-        "high",
-        "step"
-    )
+    named_children = ("low", "high", "step")
+    getLow = ExpressionChildrenHavingBase.childGetter("low")
+    getHigh = ExpressionChildrenHavingBase.childGetter("high")
+    getStep = ExpressionChildrenHavingBase.childGetter("step")
 
     def __init__(self, low, high, step, source_ref):
-        ExpressionBuiltinXrangeBase.__init__(
-            self,
-            values     = {
-                "low"  : low,
-                "high" : high,
-                "step" : step
-            },
-            source_ref = source_ref
+        ExpressionChildrenHavingBase.__init__(
+            self, values={"low": low, "high": high, "step": step}, source_ref=source_ref
         )
 
     def computeExpression(self, trace_collection):
-        low  = self.getLow()
+        low = self.getLow()
         high = self.getHigh()
         step = self.getStep()
 
         return self.computeBuiltinSpec(
-            trace_collection = trace_collection,
-            given_values     = (
-                low,
-                high,
-                step
-            )
+            trace_collection=trace_collection, given_values=(low, high, step)
         )
 
-    getLow  = ExpressionChildrenHavingBase.childGetter("low")
-    getHigh = ExpressionChildrenHavingBase.childGetter("high")
-    getStep = ExpressionChildrenHavingBase.childGetter("step")
-
     def getIterationLength(self):
-        low  = self.getLow()
+        low = self.getLow()
         high = self.getHigh()
         step = self.getStep()
 
@@ -704,7 +650,7 @@ class ExpressionBuiltinXrange3(ExpressionBuiltinXrangeBase):
         return int(estimate)
 
     def getIterationValue(self, element_index):
-        low  = self.getLow().getIntegerValue()
+        low = self.getLow().getIntegerValue()
 
         if low is None:
             return None
@@ -721,28 +667,15 @@ class ExpressionBuiltinXrange3(ExpressionBuiltinXrangeBase):
         if result >= high:
             return None
         else:
-            return makeConstantReplacementNode(
-                constant = result,
-                node     = self
-            )
+            return makeConstantReplacementNode(constant=result, node=self)
 
 
 def makeExpressionBuiltinXrange(low, high, step, source_ref):
     if high is None:
-        return ExpressionBuiltinXrange1(
-            low        = low,
-            source_ref = source_ref
-        )
+        return ExpressionBuiltinXrange1(low=low, source_ref=source_ref)
     elif step is None:
-        return ExpressionBuiltinXrange2(
-            low        = low,
-            high       = high,
-            source_ref = source_ref
-        )
+        return ExpressionBuiltinXrange2(low=low, high=high, source_ref=source_ref)
     else:
         return ExpressionBuiltinXrange3(
-            low        = low,
-            high       = high,
-            step       = step,
-            source_ref = source_ref
+            low=low, high=high, step=step, source_ref=source_ref
         )
