@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,47 +22,40 @@ whose implementation lives here. The creation itself also lives here.
 
 """
 
-from .ExpressionBases import ExpressionChildrenHavingBase
+from .ExpressionBases import ExpressionChildHavingBase
 from .FunctionNodes import ExpressionFunctionEntryPointBase
 
 
-class ExpressionMakeCoroutineObject(ExpressionChildrenHavingBase):
+class ExpressionMakeCoroutineObject(ExpressionChildHavingBase):
     kind = "EXPRESSION_MAKE_COROUTINE_OBJECT"
 
-    named_children = (
-        "coroutine_ref",
-    )
+    named_child = "coroutine_ref"
+    getCoroutineRef = ExpressionChildHavingBase.childGetter("coroutine_ref")
 
-    getCoroutineRef = ExpressionChildrenHavingBase.childGetter("coroutine_ref")
+    __slots__ = ("variable_closure_traces",)
 
     def __init__(self, coroutine_ref, source_ref):
         assert coroutine_ref.getFunctionBody().isExpressionCoroutineObjectBody()
 
-        ExpressionChildrenHavingBase.__init__(
-            self,
-            values     = {
-                "coroutine_ref" : coroutine_ref,
-            },
-            source_ref = source_ref
+        ExpressionChildHavingBase.__init__(
+            self, value=coroutine_ref, source_ref=source_ref
         )
 
         self.variable_closure_traces = None
 
     def getDetailsForDisplay(self):
-        return {
-            "coroutine" : self.getCoroutineRef().getFunctionBody().getCodeName()
-        }
+        return {"coroutine": self.getCoroutineRef().getFunctionBody().getCodeName()}
 
     def computeExpression(self, trace_collection):
         self.variable_closure_traces = []
 
-        for closure_variable in self.getCoroutineRef().getFunctionBody().getClosureVariables():
+        for closure_variable in (
+            self.getCoroutineRef().getFunctionBody().getClosureVariables()
+        ):
             trace = trace_collection.getVariableCurrentTrace(closure_variable)
-            trace.addClosureUsage()
+            trace.addNameUsage()
 
-            self.variable_closure_traces.append(
-                (closure_variable, trace)
-            )
+            self.variable_closure_traces.append((closure_variable, trace))
 
         # TODO: Coroutine body may know something too.
         return self, None, None
@@ -82,15 +75,16 @@ class ExpressionCoroutineObjectBody(ExpressionFunctionEntryPointBase):
 
     qualname_setup = None
 
-    def __init__(self, provider, name, code_object, flags, source_ref):
+    def __init__(self, provider, name, code_object, flags, auto_release, source_ref):
         ExpressionFunctionEntryPointBase.__init__(
             self,
-            provider    = provider,
-            name        = name,
-            code_object = code_object,
-            code_prefix = "coroutine",
-            flags       = flags,
-            source_ref  = source_ref
+            provider=provider,
+            name=name,
+            code_object=code_object,
+            code_prefix="coroutine",
+            flags=flags,
+            auto_release=auto_release,
+            source_ref=source_ref,
         )
 
         self.needs_generator_return_exit = False
@@ -99,10 +93,7 @@ class ExpressionCoroutineObjectBody(ExpressionFunctionEntryPointBase):
         return self.name
 
     def markAsNeedsGeneratorReturnHandling(self, value):
-        self.needs_generator_return_exit = max(
-            self.needs_generator_return_exit,
-            value
-        )
+        self.needs_generator_return_exit = max(self.needs_generator_return_exit, value)
 
     def needsGeneratorReturnHandling(self):
         return self.needs_generator_return_exit == 2
@@ -119,18 +110,17 @@ class ExpressionCoroutineObjectBody(ExpressionFunctionEntryPointBase):
         return False
 
 
-class ExpressionAsyncWait(ExpressionChildrenHavingBase):
+class ExpressionAsyncWait(ExpressionChildHavingBase):
     kind = "EXPRESSION_ASYNC_WAIT"
 
-    named_children = ("expression",)
+    named_child = "expression"
+    getValue = ExpressionChildHavingBase.childGetter("expression")
+
+    __slots__ = ("exception_preserving",)
 
     def __init__(self, expression, source_ref):
-        ExpressionChildrenHavingBase.__init__(
-            self,
-            values     = {
-                "expression" : expression
-            },
-            source_ref = source_ref
+        ExpressionChildHavingBase.__init__(
+            self, value=expression, source_ref=source_ref
         )
 
         self.exception_preserving = False
@@ -146,8 +136,6 @@ class ExpressionAsyncWait(ExpressionChildrenHavingBase):
 
         return self, None, None
 
-    getValue = ExpressionChildrenHavingBase.childGetter("expression")
-
 
 class ExpressionAsyncWaitEnter(ExpressionAsyncWait):
     kind = "EXPRESSION_ASYNC_WAIT_ENTER"
