@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,37 +26,32 @@ from nuitka.Builtins import calledWithBuiltinArgumentNamesDecorator
 from nuitka.PythonVersions import python_version
 
 from .ExpressionBases import ExpressionChildrenHavingBase
-from .NodeBases import StatementChildrenHavingBase
+from .NodeBases import StatementChildHavingBase, StatementChildrenHavingBase
 from .NodeMakingHelpers import (
     convertNoneConstantToNone,
-    makeStatementOnlyNodesFromExpressions
+    makeStatementOnlyNodesFromExpressions,
 )
 
 
 class ExpressionBuiltinEval(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_EVAL"
 
-    named_children = (
-        "source",
-        "globals",
-        "locals"
-    )
+    named_children = ("source", "globals", "locals")
+    getSourceCode = ExpressionChildrenHavingBase.childGetter("source")
+    getGlobals = ExpressionChildrenHavingBase.childGetter("globals")
+    getLocals = ExpressionChildrenHavingBase.childGetter("locals")
 
     def __init__(self, source_code, globals_arg, locals_arg, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
-            values     = {
-                "source"  : source_code,
-                "globals" : globals_arg,
-                "locals"  : locals_arg,
+            values={
+                "source": source_code,
+                "globals": globals_arg,
+                "locals": locals_arg,
             },
-            source_ref = source_ref
+            source_ref=source_ref,
         )
 
-    getSourceCode = ExpressionChildrenHavingBase.childGetter("source")
-    getGlobals = ExpressionChildrenHavingBase.childGetter("globals")
-    getLocals = ExpressionChildrenHavingBase.childGetter("locals")
-
     def computeExpression(self, trace_collection):
         # TODO: Attempt for constant values to do it.
         return self, None, None
@@ -64,21 +59,19 @@ class ExpressionBuiltinEval(ExpressionChildrenHavingBase):
 
 # Note: Python3 only so far.
 if python_version >= 300:
+
     class ExpressionBuiltinExec(ExpressionBuiltinEval):
         kind = "EXPRESSION_BUILTIN_EXEC"
 
         def __init__(self, source_code, globals_arg, locals_arg, source_ref):
             ExpressionBuiltinEval.__init__(
                 self,
-                source_code = source_code,
-                globals_arg = globals_arg,
-                locals_arg  = locals_arg,
-                source_ref  = source_ref
+                source_code=source_code,
+                globals_arg=globals_arg,
+                locals_arg=locals_arg,
+                source_ref=source_ref,
             )
 
-        def needsLocalsDict(self):
-            return True
-
         def computeExpression(self, trace_collection):
             # TODO: Attempt for constant values to do it.
             return self, None, None
@@ -86,20 +79,27 @@ if python_version >= 300:
         def computeExpressionDrop(self, statement, trace_collection):
             if self.getParentVariableProvider().isEarlyClosure():
                 result = StatementExec(
-                    source_code = self.getSourceCode(),
-                    globals_arg = self.getGlobals(),
-                    locals_arg  = self.getLocals(),
-                    source_ref  = self.getSourceReference()
+                    source_code=self.getSourceCode(),
+                    globals_arg=self.getGlobals(),
+                    locals_arg=self.getLocals(),
+                    source_ref=self.getSourceReference(),
                 )
 
-                return result, "new_statements", """\
-Replaced built-in exec call to exec statement in early closure context."""
+                del self.parent
+
+                return (
+                    result,
+                    "new_statements",
+                    """\
+Replaced built-in exec call to exec statement in early closure context.""",
+                )
             else:
                 return statement, None, None
 
 
 # Note: Python2 only
 if python_version < 300:
+
     class ExpressionBuiltinExecfile(ExpressionBuiltinEval):
         kind = "EXPRESSION_BUILTIN_EXECFILE"
 
@@ -108,15 +108,12 @@ if python_version < 300:
         def __init__(self, source_code, globals_arg, locals_arg, source_ref):
             ExpressionBuiltinEval.__init__(
                 self,
-                source_code = source_code,
-                globals_arg = globals_arg,
-                locals_arg  = locals_arg,
-                source_ref  = source_ref
+                source_code=source_code,
+                globals_arg=globals_arg,
+                locals_arg=locals_arg,
+                source_ref=source_ref,
             )
 
-        def needsLocalsDict(self):
-            return True
-
         def computeExpressionDrop(self, statement, trace_collection):
             # In this case, the copy-back must be done and will only be done
             # correctly by the code for exec statements.
@@ -124,14 +121,20 @@ if python_version < 300:
 
             if provider.isExpressionClassBody():
                 result = StatementExec(
-                    source_code = self.getSourceCode(),
-                    globals_arg = self.getGlobals(),
-                    locals_arg  = self.getLocals(),
-                    source_ref  = self.getSourceReference()
+                    source_code=self.getSourceCode(),
+                    globals_arg=self.getGlobals(),
+                    locals_arg=self.getLocals(),
+                    source_ref=self.getSourceReference(),
                 )
 
-                return result, "new_statements", """\
-Changed 'execfile' with unused result to 'exec' on class level."""
+                del self.parent
+
+                return (
+                    result,
+                    "new_statements",
+                    """\
+Changed 'execfile' with unused result to 'exec' on class level.""",
+                )
             else:
                 return statement, None, None
 
@@ -139,21 +142,20 @@ Changed 'execfile' with unused result to 'exec' on class level."""
 class StatementExec(StatementChildrenHavingBase):
     kind = "STATEMENT_EXEC"
 
-    named_children = (
-        "source",
-        "globals",
-        "locals"
-    )
+    named_children = ("source", "globals", "locals")
+    getSourceCode = StatementChildrenHavingBase.childGetter("source")
+    getGlobals = StatementChildrenHavingBase.childGetter("globals")
+    getLocals = StatementChildrenHavingBase.childGetter("locals")
 
     def __init__(self, source_code, globals_arg, locals_arg, source_ref):
         StatementChildrenHavingBase.__init__(
             self,
-            values     = {
-                "globals" : globals_arg,
-                "locals"  : locals_arg,
-                "source"  : source_code
+            values={
+                "globals": globals_arg,
+                "locals": locals_arg,
+                "source": source_code,
             },
-            source_ref = source_ref,
+            source_ref=source_ref,
         )
 
     def setChild(self, name, value):
@@ -162,23 +164,11 @@ class StatementExec(StatementChildrenHavingBase):
 
         return StatementChildrenHavingBase.setChild(self, name, value)
 
-    getSourceCode = StatementChildrenHavingBase.childGetter("source")
-    getGlobals = StatementChildrenHavingBase.childGetter("globals")
-    getLocals = StatementChildrenHavingBase.childGetter("locals")
-
-    def needsLocalsDict(self):
-        return self.getLocals().mayBeNone()
-
     def computeStatement(self, trace_collection):
-        trace_collection.onExpression(
-            expression = self.getSourceCode()
-        )
-        source_code = self.getSourceCode()
+        source_code = trace_collection.onExpression(expression=self.getSourceCode())
 
         if source_code.mayRaiseException(BaseException):
-            trace_collection.onExceptionRaiseExit(
-                BaseException
-            )
+            trace_collection.onExceptionRaiseExit(BaseException)
 
         if source_code.willRaiseException(BaseException):
             result = source_code
@@ -187,67 +177,48 @@ class StatementExec(StatementChildrenHavingBase):
                 result,
                 "new_raise",
                 """\
-Exec statement raises implicitly when determining source code argument."""
+Exec statement raises implicitly when determining source code argument.""",
             )
 
-        trace_collection.onExpression(
-            expression = self.getGlobals(),
-            allow_none = True
+        globals_arg = trace_collection.onExpression(
+            expression=self.getGlobals(), allow_none=True
         )
-        globals_arg = self.getGlobals()
 
         if globals_arg is not None and globals_arg.mayRaiseException(BaseException):
-            trace_collection.onExceptionRaiseExit(
-                BaseException
-            )
+            trace_collection.onExceptionRaiseExit(BaseException)
 
-        if globals_arg is not None and \
-           globals_arg.willRaiseException(BaseException):
+        if globals_arg is not None and globals_arg.willRaiseException(BaseException):
             result = makeStatementOnlyNodesFromExpressions(
-                expressions = (
-                    source_code,
-                    globals_arg
-                )
+                expressions=(source_code, globals_arg)
             )
 
             return (
                 result,
                 "new_raise",
                 """\
-Exec statement raises implicitly when determining globals argument."""
+Exec statement raises implicitly when determining globals argument.""",
             )
 
-        trace_collection.onExpression(
-            expression = self.getLocals(),
-            allow_none = True
+        locals_arg = trace_collection.onExpression(
+            expression=self.getLocals(), allow_none=True
         )
-        locals_arg = self.getLocals()
 
         if locals_arg is not None and locals_arg.mayRaiseException(BaseException):
-            trace_collection.onExceptionRaiseExit(
-                BaseException
-            )
+            trace_collection.onExceptionRaiseExit(BaseException)
 
-        if locals_arg is not None and \
-           locals_arg.willRaiseException(BaseException):
+        if locals_arg is not None and locals_arg.willRaiseException(BaseException):
             result = makeStatementOnlyNodesFromExpressions(
-                expressions = (
-                    source_code,
-                    globals_arg,
-                    locals_arg
-                )
+                expressions=(source_code, globals_arg, locals_arg)
             )
 
             return (
                 result,
                 "new_raise",
                 """\
-Exec statement raises implicitly when determining locals argument."""
+Exec statement raises implicitly when determining locals argument.""",
             )
 
-        trace_collection.onExceptionRaiseExit(
-            BaseException
-        )
+        trace_collection.onExceptionRaiseExit(BaseException)
 
         str_value = self.getSourceCode().getStrValue()
 
@@ -258,54 +229,55 @@ Exec statement raises implicitly when determining locals argument."""
             # TODO: This needs to be re-done.
             exec_body = None
 
-            return (
-                exec_body,
-                "new_statements",
-                "In-lined constant exec statement."
-            )
+            return (exec_body, "new_statements", "In-lined constant exec statement.")
 
         return self, None, None
 
 
-class StatementLocalsDictSync(StatementChildrenHavingBase):
+class StatementLocalsDictSync(StatementChildHavingBase):
     kind = "STATEMENT_LOCALS_DICT_SYNC"
 
-    named_children = (
-        "locals",
-    )
+    named_child = "locals"
+    getLocals = StatementChildHavingBase.childGetter("locals")
+
+    __slots__ = ("locals_scope", "previous_traces", "variable_traces")
 
     @calledWithBuiltinArgumentNamesDecorator
-    def __init__(self, locals_arg, source_ref):
-        StatementChildrenHavingBase.__init__(
-            self,
-            values     = {
-                "locals" : locals_arg,
-            },
-            source_ref = source_ref,
-        )
+    def __init__(self, locals_scope, locals_arg, source_ref):
+        StatementChildHavingBase.__init__(self, value=locals_arg, source_ref=source_ref)
 
         self.previous_traces = None
         self.variable_traces = None
 
+        self.locals_scope = locals_scope
+
+    def getDetails(self):
+        return {"locals_scope": self.locals_scope}
+
+    def getPreviousVariablesTraces(self):
+        return self.previous_traces
+
     def computeStatement(self, trace_collection):
         result, change_tags, change_desc = self.computeStatementSubExpressions(
-            trace_collection = trace_collection
+            trace_collection=trace_collection
         )
 
         if result is not self:
             return result, change_tags, change_desc
 
-        if self.getParentVariableProvider().isCompiledPythonModule():
+        provider = self.getParentVariableProvider()
+        if provider.isCompiledPythonModule():
+            return None, "new_statements", "Removed sync back to locals without locals."
+
+        self.previous_traces = trace_collection.onLocalsUsage(self.locals_scope)
+        if not self.previous_traces:
             return None, "new_statements", "Removed sync back to locals without locals."
 
-        self.previous_traces = trace_collection.onLocalsUsage(self.getParentVariableProvider())
         trace_collection.removeAllKnowledge()
-        self.variable_traces = trace_collection.onLocalsUsage(self.getParentVariableProvider())
+        self.variable_traces = trace_collection.onLocalsUsage(self.locals_scope)
 
         return self, None, None
 
-    getLocals = ExpressionChildrenHavingBase.childGetter("locals")
-
     def mayRaiseException(self, exception_type):
         return False
 
@@ -313,30 +285,7 @@ class StatementLocalsDictSync(StatementChildrenHavingBase):
 class ExpressionBuiltinCompile(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_COMPILE"
 
-    named_children = (
-        "source",
-        "filename",
-        "mode",
-        "flags",
-        "dont_inherit",
-        "optimize"
-    )
-
-    def __init__(self, source_code, filename, mode, flags, dont_inherit,
-                 optimize, source_ref):
-        ExpressionChildrenHavingBase.__init__(
-            self,
-            values     = {
-                "source"       : source_code,
-                "filename"     : filename,
-                "mode"         : mode,
-                "flags"        : flags,
-                "dont_inherit" : dont_inherit,
-                "optimize"     : optimize
-            },
-            source_ref = source_ref
-        )
-
+    named_children = ("source", "filename", "mode", "flags", "dont_inherit", "optimize")
     getSourceCode = ExpressionChildrenHavingBase.childGetter("source")
     getFilename = ExpressionChildrenHavingBase.childGetter("filename")
     getMode = ExpressionChildrenHavingBase.childGetter("mode")
@@ -344,6 +293,22 @@ class ExpressionBuiltinCompile(ExpressionChildrenHavingBase):
     getDontInherit = ExpressionChildrenHavingBase.childGetter("dont_inherit")
     getOptimize = ExpressionChildrenHavingBase.childGetter("optimize")
 
+    def __init__(
+        self, source_code, filename, mode, flags, dont_inherit, optimize, source_ref
+    ):
+        ExpressionChildrenHavingBase.__init__(
+            self,
+            values={
+                "source": source_code,
+                "filename": filename,
+                "mode": mode,
+                "flags": flags,
+                "dont_inherit": dont_inherit,
+                "optimize": optimize,
+            },
+            source_ref=source_ref,
+        )
+
     def computeExpression(self, trace_collection):
         trace_collection.onExceptionRaiseExit(BaseException)
 
