@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -59,7 +59,7 @@ class ExpressionMakeGeneratorObject(ExpressionChildHavingBase):
             self.getGeneratorRef().getFunctionBody().getClosureVariables()
         ):
             trace = trace_collection.getVariableCurrentTrace(closure_variable)
-            trace.addClosureUsage()
+            trace.addNameUsage()
 
             self.variable_closure_traces.append((closure_variable, trace))
 
@@ -126,14 +126,15 @@ class StatementGeneratorReturn(StatementReturn):
         StatementReturn.__init__(self, expression=expression, source_ref=source_ref)
 
     def computeStatement(self, trace_collection):
-        trace_collection.onExpression(self.getExpression())
-        expression = self.getExpression()
+        expression = trace_collection.onExpression(self.getExpression())
 
         if expression.mayRaiseException(BaseException):
             trace_collection.onExceptionRaiseExit(BaseException)
 
         if expression.willRaiseException(BaseException):
-            from .NodeMakingHelpers import makeStatementExpressionOnlyReplacementNode
+            from .NodeMakingHelpers import (
+                makeStatementExpressionOnlyReplacementNode,
+            )
 
             result = makeStatementExpressionOnlyReplacementNode(
                 expression=expression, node=self
