@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -30,32 +30,34 @@ from logging import warning
 
 from nuitka.__past__ import (  # pylint: disable=I0021,redefined-builtin
     long,
-    unicode
+    unicode,
 )
 from nuitka.Builtins import calledWithBuiltinArgumentNamesDecorator
 from nuitka.importing.Importing import (
     findModule,
-    getModuleNameAndKindFromFilename
+    getModuleNameAndKindFromFilename,
 )
 from nuitka.importing.Recursion import decideRecursion, recurseTo
 from nuitka.importing.Whitelisting import getModuleWhiteList
+from nuitka.ModuleRegistry import getUncompiledModule
 from nuitka.utils.FileOperations import relpath
+from nuitka.utils.ModuleNames import ModuleName
 
 from .ExpressionBases import (
     ExpressionBase,
     ExpressionChildHavingBase,
-    ExpressionChildrenHavingBase
+    ExpressionChildrenHavingBase,
 )
 from .LocalsScopes import GlobalsDictHandle
 from .NodeBases import StatementChildHavingBase
-from .shapes.BuiltinTypeShapes import ShapeTypeBuiltinModule, ShapeTypeModule
+from .shapes.BuiltinTypeShapes import tshape_module, tshape_module_builtin
 
 
 class ExpressionImportModuleHard(ExpressionBase):
-    """ Hard coded import names, e.g. of "__future__"
+    """Hard coded import names, e.g. of "__future__"
 
-        These are directly created for some Python mechanics, but also due to
-        compile time optimization for imports of statically known modules.
+    These are directly created for some Python mechanics, but also due to
+    compile time optimization for imports of statically known modules.
     """
 
     kind = "EXPRESSION_IMPORT_MODULE_HARD"
@@ -63,10 +65,7 @@ class ExpressionImportModuleHard(ExpressionBase):
     __slots__ = ("module_name",)
 
     def __init__(self, module_name, source_ref):
-        ExpressionBase.__init__(
-            self,
-            source_ref = source_ref
-        )
+        ExpressionBase.__init__(self, source_ref=source_ref)
 
         self.module_name = module_name
 
@@ -74,9 +73,7 @@ class ExpressionImportModuleHard(ExpressionBase):
         del self.parent
 
     def getDetails(self):
-        return {
-            "module_name" : self.module_name
-        }
+        return {"module_name": self.module_name}
 
     def getModuleName(self):
         return self.module_name
@@ -97,9 +94,9 @@ class ExpressionImportModuleHard(ExpressionBase):
 
 
 class ExpressionImportModuleNameHard(ExpressionBase):
-    """ Hard coded import names, e.g. of "os.path.dirname"
+    """Hard coded import names, e.g. of "os.path.dirname"
 
-        These are directly created for some Python mechanics.
+    These are directly created for some Python mechanics.
     """
 
     kind = "EXPRESSION_IMPORT_MODULE_NAME_HARD"
@@ -107,10 +104,7 @@ class ExpressionImportModuleNameHard(ExpressionBase):
     __slots__ = "module_name", "import_name"
 
     def __init__(self, module_name, import_name, source_ref):
-        ExpressionBase.__init__(
-            self,
-            source_ref = source_ref
-        )
+        ExpressionBase.__init__(self, source_ref=source_ref)
 
         self.module_name = module_name
         self.import_name = import_name
@@ -119,10 +113,7 @@ class ExpressionImportModuleNameHard(ExpressionBase):
         del self.parent
 
     def getDetails(self):
-        return {
-            "module_name" : self.module_name,
-            "import_name" : self.import_name
-        }
+        return {"module_name": self.module_name, "import_name": self.import_name}
 
     def getModuleName(self):
         return self.module_name
@@ -150,25 +141,27 @@ class ExpressionImportModuleNameHard(ExpressionBase):
 class ExpressionBuiltinImport(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_IMPORT"
 
-    named_children = (
-        "name", "globals", "locals", "fromlist", "level"
-    )
+    named_children = ("name", "globals", "locals", "fromlist", "level")
+    getImportName = ExpressionChildrenHavingBase.childGetter("name")
+    getFromList = ExpressionChildrenHavingBase.childGetter("fromlist")
+    getGlobals = ExpressionChildrenHavingBase.childGetter("globals")
+    getLocals = ExpressionChildrenHavingBase.childGetter("locals")
+    getLevel = ExpressionChildrenHavingBase.childGetter("level")
 
     _warned_about = set()
 
     @calledWithBuiltinArgumentNamesDecorator
-    def __init__(self, name, globals_arg, locals_arg, fromlist, level,
-                source_ref):
+    def __init__(self, name, globals_arg, locals_arg, fromlist, level, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
-            values     = {
-                "name"     : name,
-                "globals"  : globals_arg,
-                "locals"   : locals_arg,
-                "fromlist" : fromlist,
-                "level"    : level
+            values={
+                "name": name,
+                "globals": globals_arg,
+                "locals": locals_arg,
+                "fromlist": fromlist,
+                "level": level,
             },
-            source_ref = source_ref
+            source_ref=source_ref,
         )
 
         self.recurse_attempted = False
@@ -186,59 +179,61 @@ class ExpressionBuiltinImport(ExpressionChildrenHavingBase):
 
         self.finding = None
 
-        self.type_shape = ShapeTypeModule
+        self.type_shape = tshape_module
 
         self.builtin_module = None
 
-    getImportName = ExpressionChildrenHavingBase.childGetter("name")
-    getFromList = ExpressionChildrenHavingBase.childGetter("fromlist")
-    getGlobals = ExpressionChildrenHavingBase.childGetter("globals")
-    getLocals = ExpressionChildrenHavingBase.childGetter("locals")
-    getLevel = ExpressionChildrenHavingBase.childGetter("level")
-
     def _consider(self, trace_collection, module_filename, module_package):
-        assert module_package is None or \
-              (type(module_package) is str and module_package != ""), repr(module_package)
+        assert module_package is None or (
+            type(module_package) is ModuleName and module_package != ""
+        ), repr(module_package)
 
         module_filename = os.path.normpath(module_filename)
 
         module_name, module_kind = getModuleNameAndKindFromFilename(module_filename)
 
         if module_kind is not None:
+            module_fullpath = ModuleName.makeModuleNameInPackage(
+                module_name, module_package
+            )
+
             decision, reason = decideRecursion(
-                module_filename = module_filename,
-                module_name     = module_name,
-                module_package  = module_package,
-                module_kind     = module_kind
+                module_filename=module_filename,
+                module_name=module_fullpath,
+                module_kind=module_kind,
             )
 
             if decision:
                 module_relpath = relpath(module_filename)
 
                 imported_module, added_flag = recurseTo(
-                    module_package  = module_package,
-                    module_filename = module_filename,
-                    module_relpath  = module_relpath,
-                    module_kind     = module_kind,
-                    reason          = reason
+                    module_package=module_package,
+                    module_filename=module_filename,
+                    module_relpath=module_relpath,
+                    module_kind=module_kind,
+                    reason=reason,
                 )
 
                 if added_flag:
                     trace_collection.signalChange(
                         "new_code",
                         imported_module.getSourceReference(),
-                        "Recursed to module."
+                        "Recursed to module.",
                     )
 
                 return imported_module
-            elif decision is None and module_kind == "py":
-                if module_package is None:
-                    module_fullpath = module_name
-                else:
-                    module_fullpath = module_package + '.' + module_name
+            elif decision is False and module_kind == "py":
+                uncompiled_module = getUncompiledModule(
+                    module_fullpath, module_filename
+                )
 
-                if module_filename not in self._warned_about and \
-                   module_fullpath not in getModuleWhiteList():
+                if uncompiled_module is not None:
+                    return uncompiled_module
+            elif decision is None and module_kind == "py":
+                if (
+                    module_filename not in self._warned_about
+                    and module_fullpath not in getModuleWhiteList()
+                ):
                     self._warned_about.add(module_filename)
 
                     warning(
@@ -247,10 +242,8 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
 --nofollow-imports (do not warn), \
 --follow-imports (recurse to all), \
 --nofollow-import-to=%(full_path)s (ignore it), \
---follow-import-to=%(full_path)s (recurse to it) to change.""" % {
-                            "full_path" : module_fullpath,
-                            "filename"  : module_filename
-                        }
+--follow-import-to=%(full_path)s (recurse to it) to change."""
+                        % {"full_path": module_fullpath, "filename": module_filename}
                     )
 
     def _attemptRecursion(self, trace_collection, module_name):
@@ -258,10 +251,9 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
 
         parent_module = self.getParentModule()
 
-        if parent_module.isCompiledPythonPackage():
-            parent_package = parent_module.getFullName()
-        else:
-            parent_package = self.getParentModule().getPackage()
+        parent_package = parent_module.getFullName()
+        if not parent_module.isCompiledPythonPackage():
+            parent_package = parent_package.getPackageName()
 
         level = self.getLevel()
 
@@ -277,24 +269,24 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
             return
 
         module_package, module_filename, self.finding = findModule(
-            importing      = self,
-            module_name    = module_name,
-            parent_package = parent_package,
-            level          = level,
-            warn           = True
+            importing=self,
+            module_name=ModuleName(module_name),
+            parent_package=parent_package,
+            level=level,
+            warn=True,
         )
 
         if module_filename is not None:
             imported_module = self._consider(
-                trace_collection = trace_collection,
-                module_filename  = module_filename,
-                module_package   = module_package
+                trace_collection=trace_collection,
+                module_filename=module_filename,
+                module_package=module_package,
             )
 
             if imported_module is not None:
                 self.imported_module_desc = (
                     imported_module.getFullName(),
-                    imported_module.getFilename()
+                    imported_module.getFilename(),
                 )
 
                 import_list = self.getFromList()
@@ -306,51 +298,55 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
                     if type(import_list) not in (tuple, list):
                         import_list = None
 
-                if import_list and \
-                   imported_module.isCompiledPythonPackage():
+                if import_list and imported_module.isCompiledPythonPackage():
                     for import_item in import_list:
-                        if import_item == '*':
+                        if import_item == "*":
                             continue
 
                         module_package, module_filename, _finding = findModule(
-                            importing      = self,
-                            module_name    = import_item,
-                            parent_package = imported_module.getFullName(),
-                            level          = -1, # Relative import, so child is used.
-                            warn           = False
+                            importing=self,
+                            module_name=ModuleName(import_item),
+                            parent_package=imported_module.getFullName(),
+                            level=-1,  # Relative import, so child is used.
+                            warn=False,
                         )
 
                         if module_filename is not None:
                             sub_imported_module = self._consider(
-                                trace_collection = trace_collection,
-                                module_filename  = module_filename,
-                                module_package   = module_package
+                                trace_collection=trace_collection,
+                                module_filename=module_filename,
+                                module_package=module_package,
                             )
 
                             if sub_imported_module is not None:
                                 self.import_list_modules_desc.append(
                                     (
                                         sub_imported_module.getFullName(),
-                                        sub_imported_module.getFilename()
+                                        sub_imported_module.getFilename(),
                                     )
                                 )
         else:
-            while '.' in module_name:
-                module_name = '.'.join(module_name.split('.')[:-1])
+            module_name = ModuleName(module_name)
+
+            while True:
+                module_name = module_name.getPackageName()
+
+                if module_name is None:
+                    break
 
                 module_package, module_filename, _finding = findModule(
-                    importing      = self,
-                    module_name    = module_name,
-                    parent_package = parent_package,
-                    level          = level,
-                    warn           = True
+                    importing=self,
+                    module_name=module_name,
+                    parent_package=parent_package,
+                    level=level,
+                    warn=True,
                 )
 
                 if module_filename is not None:
                     package_module = self._consider(
-                        trace_collection = trace_collection,
-                        module_filename  = module_filename,
-                        module_package   = module_package
+                        trace_collection=trace_collection,
+                        module_filename=module_filename,
+                        module_package=module_package,
                     )
 
                     if package_module is not None:
@@ -358,37 +354,29 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
                             self.package_modules_desc = []
 
                         self.package_modules_desc.append(
-                            (
-                                package_module.getFullName(),
-                                package_module.getFilename()
-                            )
+                            (package_module.getFullName(), package_module.getFilename())
                         )
 
-
     def _addUsedModules(self, trace_collection):
         if self.finding != "not-found":
             if self.imported_module_desc is not None:
                 trace_collection.onUsedModule(
-                    module_name    = self.imported_module_desc[0],
-                    module_relpath = self.imported_module_desc[1]
+                    module_name=self.imported_module_desc[0],
+                    module_relpath=self.imported_module_desc[1],
                 )
 
             for import_list_module_desc in self.import_list_modules_desc:
                 trace_collection.onUsedModule(
-                    import_list_module_desc[0],
-                    import_list_module_desc[1]
+                    import_list_module_desc[0], import_list_module_desc[1]
                 )
 
         # These are added in any case.
         if self.package_modules_desc is not None:
             for package_module_desc in self.package_modules_desc:
                 trace_collection.onUsedModule(
-                    package_module_desc[0],
-                    package_module_desc[1]
+                    package_module_desc[0], package_module_desc[1]
                 )
 
-
-
     def computeExpression(self, trace_collection):
         # TODO: In fact, if the module is not a package, we don't have to insist
         # on the "fromlist" that much, but normally it's not used for anything
@@ -417,16 +405,14 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
                 if str is not unicode and type(imported_module_name) is unicode:
                     imported_module_name = str(imported_module_name)
 
-
                 self._attemptRecursion(
-                    trace_collection = trace_collection,
-                    module_name      = imported_module_name
+                    trace_collection=trace_collection, module_name=imported_module_name
                 )
 
                 self.recurse_attempted = True
 
                 if self.finding == "built-in":
-                    self.type_shape = ShapeTypeBuiltinModule
+                    self.type_shape = tshape_module_builtin
                     self.builtin_module = __import__(imported_module_name)
 
                 self._addUsedModules(trace_collection)
@@ -434,10 +420,14 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
                 # TODO: This doesn't preserve side effects.
 
                 # Non-strings is going to raise an error.
-                new_node, change_tags, message = trace_collection.getCompileTimeComputationResult(
-                    node        = self,
-                    computation = lambda : __import__(module_name.getConstant()),
-                    description = "Replaced '__import__' call with non-string module name argument."
+                (
+                    new_node,
+                    change_tags,
+                    message,
+                ) = trace_collection.getCompileTimeComputationResult(
+                    node=self,
+                    computation=lambda: __import__(module_name.getConstant()),
+                    description="Replaced '__import__' call with non-string module name argument.",
                 )
 
                 # Must fail, must not go on when it doesn't.
@@ -473,14 +463,13 @@ class StatementImportStar(StatementChildHavingBase):
     kind = "STATEMENT_IMPORT_STAR"
 
     named_child = "module"
+    getSourceModule = StatementChildHavingBase.childGetter("module")
 
     __slots__ = ("target_scope",)
 
     def __init__(self, target_scope, module_import, source_ref):
         StatementChildHavingBase.__init__(
-            self,
-            value      = module_import,
-            source_ref = source_ref
+            self, value=module_import, source_ref=source_ref
         )
 
         self.target_scope = target_scope
@@ -489,8 +478,6 @@ class StatementImportStar(StatementChildHavingBase):
         if type(self.target_scope) is GlobalsDictHandle:
             self.target_scope.markAsEscaped()
 
-    getSourceModule = StatementChildHavingBase.childGetter("module")
-
     def getTargetDictScope(self):
         return self.target_scope
 
@@ -522,15 +509,12 @@ class ExpressionImportName(ExpressionChildHavingBase):
     kind = "EXPRESSION_IMPORT_NAME"
 
     named_child = "module"
+    getModule = ExpressionChildrenHavingBase.childGetter("module")
 
     __slots__ = ("import_name", "level")
 
     def __init__(self, module, import_name, level, source_ref):
-        ExpressionChildHavingBase.__init__(
-            self,
-            value      = module,
-            source_ref = source_ref
-        )
+        ExpressionChildHavingBase.__init__(self, value=module, source_ref=source_ref)
 
         self.import_name = import_name
         self.level = level
@@ -544,28 +528,16 @@ class ExpressionImportName(ExpressionChildHavingBase):
         return self.level
 
     def getDetails(self):
-        return {
-            "import_name" : self.getImportName(),
-            "level"       : self.level
-        }
-
-    def getDetail(self):
-        return "import %s from %s" % (
-            self.getImportName(),
-            self.getModule().getModuleName()
-        )
-
-    getModule = ExpressionChildrenHavingBase.childGetter("module")
+        return {"import_name": self.getImportName(), "level": self.level}
 
     def computeExpression(self, trace_collection):
         return self.getModule().computeExpressionImportName(
-            import_node      = self,
-            import_name      = self.import_name,
-            trace_collection = trace_collection
+            import_node=self,
+            import_name=self.import_name,
+            trace_collection=trace_collection,
         )
 
     def mayRaiseException(self, exception_type):
         return self.getModule().mayRaiseExceptionImportName(
-            exception_type = exception_type,
-            import_name    = self.import_name
+            exception_type=exception_type, import_name=self.import_name
         )
