@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -27,65 +27,48 @@ from nuitka import Options, Variables
 from nuitka.containers.oset import OrderedSet
 from nuitka.importing.Importing import (
     findModule,
-    getModuleNameAndKindFromFilename
+    getModuleNameAndKindFromFilename,
 )
 from nuitka.importing.Recursion import decideRecursion, recurseTo
 from nuitka.ModuleRegistry import getModuleByName, getOwnerFromCodeName
 from nuitka.optimizations.TraceCollections import TraceCollectionModule
 from nuitka.PythonVersions import python_version
-from nuitka.SourceCodeReferences import SourceCodeReference, fromFilename
+from nuitka.SourceCodeReferences import fromFilename
+from nuitka.tree.SourceReading import readSourceCodeFromFilename
 from nuitka.utils.CStrings import encodePythonIdentifierToC
-from nuitka.utils.FileOperations import relpath
+from nuitka.utils.FileOperations import getFileContentByLine, relpath
+from nuitka.utils.ModuleNames import ModuleName
 
 from .Checkers import checkStatementsSequenceOrNone
-from .FutureSpecs import FutureSpec, fromFlags
+from .FutureSpecs import fromFlags
 from .IndicatorMixins import EntryPointMixin, MarkNeedsAnnotationsMixin
-from .LocalsScopes import getLocalsDictHandle, setLocalsDictType
+from .LocalsScopes import getLocalsDictHandle
 from .NodeBases import (
     ChildrenHavingMixin,
     ClosureGiverNodeMixin,
     NodeBase,
     extractKindAndArgsFromXML,
-    fromXML
+    fromXML,
 )
 
 
 class PythonModuleBase(NodeBase):
     # Base classes can be abstract, pylint: disable=abstract-method
 
-    __slots__ = "name", "package_name", "package"
+    __slots__ = ("module_name",)
 
-    def __init__(self, name, package_name, source_ref):
-        assert type(name) is str, type(name)
-        assert '.' not in name, name
-        assert package_name is None or \
-               (type(package_name) is str and package_name != "")
+    def __init__(self, module_name, source_ref):
+        assert type(module_name) is ModuleName, module_name
 
-        NodeBase.__init__(
-            self,
-            source_ref = source_ref
-        )
+        NodeBase.__init__(self, source_ref=source_ref)
 
-        self.name = name
-        self.package_name = package_name
-        self.package = None
+        self.module_name = module_name
 
     def getDetails(self):
-        return {
-            "name" : self.name
-        }
-
-    def getName(self):
-        return self.name
-
-    def getPackage(self):
-        return self.package_name
+        return {"module_name": self.module_name}
 
     def getFullName(self):
-        if self.package_name:
-            return self.package_name + '.' + self.getName()
-        else:
-            return self.getName()
+        return self.module_name
 
     @staticmethod
     def isMainModule():
@@ -95,67 +78,66 @@ class PythonModuleBase(NodeBase):
     def isTopModule():
         return False
 
-    @staticmethod
-    def isInternalModule():
-        return False
-
     def attemptRecursion(self):
-        # Make sure the package is recursed to.
+        # Make sure the package is recursed to if any
+        package_name = self.module_name.getPackageName()
+        if package_name is None:
+            return ()
 
         # Return the list of newly added modules.
+
         result = []
+        package = getModuleByName(package_name)
 
-        if self.package_name is not None and self.package is None:
-            self.package = getModuleByName(self.package_name)
-
-        if self.package_name is not None and self.package is None:
+        if package_name is not None and package is None:
             package_package, package_filename, finding = findModule(
-                importing      = self,
-                module_name    = self.package_name,
-                parent_package = None,
-                level          = 1,
-                warn           = python_version < 300
+                importing=self,
+                module_name=package_name,
+                parent_package=None,
+                level=1,
+                warn=python_version < 300,
             )
 
             # TODO: Temporary, if we can't find the package for Python3.3 that
             # is semi-OK, maybe.
             if python_version >= 300 and not package_filename:
-                return []
+                return ()
 
-            if self.package_name == "uniconvertor.app.modules":
-                return []
+            if package_name == "uniconvertor.app.modules":
+                return ()
 
-            assert package_filename is not None, (self.package_name, finding)
+            assert package_filename is not None, (package_name, finding)
 
-            _package_name, package_kind = getModuleNameAndKindFromFilename(package_filename)
+            _package_name, package_kind = getModuleNameAndKindFromFilename(
+                package_filename
+            )
             # assert _package_name == self.package_name, (package_filename, _package_name, self.package_name)
 
             decision, _reason = decideRecursion(
-                module_filename = package_filename,
-                module_name     = self.package_name,
-                module_package  = package_package,
-                module_kind     = package_kind
+                module_filename=package_filename,
+                module_name=package_name,
+                module_kind=package_kind,
             )
 
             if decision is not None:
-                self.package, is_added = recurseTo(
-                    module_package  = package_package,
-                    module_filename = package_filename,
-                    module_relpath  = relpath(package_filename),
-                    module_kind     = "py",
-                    reason          = "Containing package of recursed module '%s'." % self.getFullName(),
+                package, is_added = recurseTo(
+                    module_package=package_package,
+                    module_filename=package_filename,
+                    module_relpath=relpath(package_filename),
+                    module_kind="py",
+                    reason="Containing package of recursed module '%s'."
+                    % self.getFullName(),
                 )
 
                 if is_added:
-                    result.append(self.package)
+                    result.append(package)
 
-        if self.package:
+        if package:
             from nuitka.ModuleRegistry import addUsedModule
 
-            addUsedModule(self.package)
+            addUsedModule(package)
 
-#            print "Recursed to package", self.package_name
-            result.extend(self.package.attemptRecursion())
+            result.extend(package.attemptRecursion())
 
         return result
 
@@ -164,8 +146,36 @@ class PythonModuleBase(NodeBase):
         return None
 
     def getCompileTimeFilename(self):
+        """The compile time filename for the module.
+
+        Returns:
+            Full path to module file at compile time.
+        Notes:
+            We are getting the absolute path here, since we do
+            not want to have to deal with resolving paths at
+            all.
+
+        """
         return os.path.abspath(self.getSourceReference().getFilename())
 
+    def getCompileTimeDirectory(self):
+        """The compile time directory for the module.
+
+        Returns:
+            Full path to module directory at compile time.
+        Notes:
+            For packages, we let the package directory be
+            the result, otherwise the containing directory
+            is the result.
+        Notes:
+            Use this to find files nearby a module, mainly
+            in plugin code.
+        """
+        result = self.getCompileTimeFilename()
+        if not os.path.isdir(result):
+            result = os.path.dirname(result)
+        return result
+
     def getRunTimeFilename(self):
         # TODO: Don't look at such things this late, push this into building.
         reference_mode = Options.getFileReferenceMode()
@@ -182,61 +192,48 @@ class PythonModuleBase(NodeBase):
             result = os.path.basename(filename)
             current = filename
 
-            levels = full_name.count('.')
+            levels = full_name.count(".")
             if self.isCompiledPythonPackage():
                 levels += 1
 
             for _i in range(levels):
                 current = os.path.dirname(current)
 
-                result = os.path.join(
-                    os.path.basename(current),
-                    result
-                )
+                result = os.path.join(os.path.basename(current), result)
 
             return result
 
 
-class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
-                           MarkNeedsAnnotationsMixin, EntryPointMixin,
-                           PythonModuleBase):
-    """ Compiled Python Module
-
-    """
+class CompiledPythonModule(
+    ChildrenHavingMixin,
+    ClosureGiverNodeMixin,
+    MarkNeedsAnnotationsMixin,
+    EntryPointMixin,
+    PythonModuleBase,
+):
+    """Compiled Python Module"""
 
     # This one has a few indicators, pylint: disable=too-many-instance-attributes
 
     kind = "COMPILED_PYTHON_MODULE"
 
-    named_children = (
-        "body",
-        "functions"
-    )
+    named_children = ("body", "functions")
+    getBody = ChildrenHavingMixin.childGetter("body")
+    setBody = ChildrenHavingMixin.childSetter("body")
+    getFunctions = ChildrenHavingMixin.childGetter("functions")
+    setFunctions = ChildrenHavingMixin.childSetter("functions")
 
-    checkers = {
-        "body": checkStatementsSequenceOrNone
-    }
+    checkers = {"body": checkStatementsSequenceOrNone}
 
-    def __init__(self, name, package_name, is_top, mode, future_spec, source_ref):
-        PythonModuleBase.__init__(
-            self,
-            name         = name,
-            package_name = package_name,
-            source_ref   = source_ref
-        )
+    def __init__(self, module_name, is_top, mode, future_spec, source_ref):
+        PythonModuleBase.__init__(self, module_name=module_name, source_ref=source_ref)
 
         ClosureGiverNodeMixin.__init__(
-            self,
-            name        = name,
-            code_prefix = "module"
+            self, name=module_name.getBasename(), code_prefix="module"
         )
 
         ChildrenHavingMixin.__init__(
-            self,
-            values = {
-                "body" : None, # delayed
-                "functions" : (),
-            },
+            self, values={"body": None, "functions": ()}  # delayed
         )
 
         MarkNeedsAnnotationsMixin.__init__(self)
@@ -252,29 +249,37 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
         self.active_functions = OrderedSet()
         self.cross_used_functions = OrderedSet()
 
+        self.used_modules = OrderedSet()
+
         # Often "None" until tree building finishes its part.
         self.future_spec = future_spec
 
-        self.module_dict_name = "globals_%s" % (
-            self.getCodeName(),
+        # The source code of the module if changed or not from disk.
+        self.source_code = None
+
+        self.module_dict_name = "globals_%s" % (self.getCodeName(),)
+
+        self.locals_scope = getLocalsDictHandle(
+            self.module_dict_name, "module_dict", self
         )
-        setLocalsDictType(self.module_dict_name, "module_dict")
 
     def getDetails(self):
         return {
-            "filename" : self.source_ref.getFilename(),
-            "package"  : self.package_name,
-            "name"     : self.name
+            "filename": self.source_ref.getFilename(),
+            "module_name": self.module_name,
         }
 
     def getDetailsForDisplay(self):
         result = self.getDetails()
 
         if self.future_spec is not None:
-            result["code_flags"] = ','.join(self.future_spec.asFlags())
+            result["code_flags"] = ",".join(self.future_spec.asFlags())
 
         return result
 
+    def getCompilationMode(self):
+        return self.mode
+
     @classmethod
     def fromXML(cls, provider, source_ref, **args):
         # Modules are not having any provider, must not be used,
@@ -288,21 +293,19 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
 
     def asGraph(self, graph, desc):
         graph = graph.add_subgraph(
-            name    = "cluster_%s" % desc,
-            comment = "Graph for %s" % self.getName()
+            name="cluster_%s" % desc, comment="Graph for %s" % self.getName()
         )
 
-#        graph.body.append("style=filled")
-#        graph.body.append("color=lightgrey")
-#        graph.body.append("label=Iteration_%d" % desc)
-
+        #        graph.body.append("style=filled")
+        #        graph.body.append("color=lightgrey")
+        #        graph.body.append("label=Iteration_%d" % desc)
 
         def makeTraceNodeName(variable, version, variable_trace):
             return "%s/ %s %s %s" % (
                 desc,
                 variable.getName(),
                 version,
-                variable_trace.__class__.__name__
+                variable_trace.__class__.__name__,
             )
 
         for function_body in self.active_functions:
@@ -310,24 +313,26 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
 
             node_names = {}
 
-            for (variable, version), variable_trace in trace_collection.getVariableTracesAll().items():
+            for (
+                (variable, version),
+                variable_trace,
+            ) in trace_collection.getVariableTracesAll().items():
                 node_name = makeTraceNodeName(variable, version, variable_trace)
 
                 node_names[variable_trace] = node_name
 
-            for (variable, version), variable_trace in trace_collection.getVariableTracesAll().items():
+            for (
+                (variable, version),
+                variable_trace,
+            ) in trace_collection.getVariableTracesAll().items():
                 node_name = node_names[variable_trace]
 
                 previous = variable_trace.getPrevious()
 
-                attrs = {
-                    "style" : "filled",
-                }
+                attrs = {"style": "filled"}
 
-                if variable_trace.hasDefiniteUsages():
+                if variable_trace.getUsageCount():
                     attrs["color"] = "blue"
-                elif variable_trace.hasPotentialUsages():
-                    attrs["color"] = "yellow"
                 else:
                     attrs["color"] = "red"
 
@@ -345,16 +350,22 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
 
         return graph
 
-    getBody = ChildrenHavingMixin.childGetter("body")
-    setBody = ChildrenHavingMixin.childSetter("body")
-
-    getFunctions = ChildrenHavingMixin.childGetter("functions")
-    setFunctions = ChildrenHavingMixin.childSetter("functions")
-
     @staticmethod
     def isCompiledPythonModule():
         return True
 
+    def getSourceCode(self):
+        if self.source_code is not None:
+            return self.source_code
+        else:
+            return readSourceCodeFromFilename(
+                module_name=self.getFullName(),
+                source_filename=self.getCompileTimeFilename(),
+            )
+
+    def setSourceCode(self, code):
+        self.source_code = code
+
     def getParent(self):
         # We have never have a parent
         return None
@@ -366,7 +377,7 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
     def hasVariableName(self, variable_name):
         return variable_name in self.variables or variable_name in self.temp_variables
 
-    def getVariables(self):
+    def getProvidedVariables(self):
         return self.variables.values()
 
     def getFilename(self):
@@ -379,17 +390,12 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
         return self.getProvidedVariable(variable_name)
 
     def getVariableForClosure(self, variable_name):
-        return self.getProvidedVariable(
-            variable_name = variable_name
-        )
+        return self.getProvidedVariable(variable_name=variable_name)
 
     def createProvidedVariable(self, variable_name):
         assert variable_name not in self.variables
 
-        result = Variables.ModuleVariable(
-            module        = self,
-            variable_name = variable_name
-        )
+        result = Variables.ModuleVariable(module=self, variable_name=variable_name)
 
         self.variables[variable_name] = result
 
@@ -420,16 +426,25 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
         self.setFunctions(functions)
 
     def startTraversal(self):
+        self.used_modules = OrderedSet()
         self.active_functions = OrderedSet()
 
+    def addUsedModule(self, key):
+        self.used_modules.add(key)
+
+    def getUsedModules(self):
+        return self.used_modules
+
     def addUsedFunction(self, function_body):
         assert function_body in self.getFunctions()
 
-        assert function_body.isExpressionFunctionBody() or \
-               function_body.isExpressionClassBody() or \
-               function_body.isExpressionGeneratorObjectBody() or \
-               function_body.isExpressionCoroutineObjectBody() or \
-               function_body.isExpressionAsyncgenObjectBody()
+        assert (
+            function_body.isExpressionFunctionBody()
+            or function_body.isExpressionClassBody()
+            or function_body.isExpressionGeneratorObjectBody()
+            or function_body.isExpressionCoroutineObjectBody()
+            or function_body.isExpressionAsyncgenObjectBody()
+        )
 
         result = function_body not in self.active_functions
         if result:
@@ -469,7 +484,7 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
 
         # There are some characters that somehow are passed to shell, by
         # Scons or unknown, so lets avoid them for now.
-        return result.replace(')',"").replace('(',"")
+        return result.replace(")", "").replace("(", "")
 
     def computeModule(self):
         old_collection = self.trace_collection
@@ -480,7 +495,7 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
 
         if module_body is not None:
             result = module_body.computeStatementsSequence(
-                trace_collection = self.trace_collection
+                trace_collection=self.trace_collection
             )
 
             if result is not module_body:
@@ -490,12 +505,14 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
 
         for new_module in new_modules:
             self.trace_collection.signalChange(
-                source_ref = new_module.getSourceReference(),
-                tags       = "new_code",
-                message    = "Recursed to module package."
+                source_ref=new_module.getSourceReference(),
+                tags="new_code",
+                message="Recursed to module package.",
             )
 
-        self.trace_collection.updateVariablesFromCollection(old_collection)
+        self.trace_collection.updateVariablesFromCollection(
+            old_collection, self.source_ref
+        )
 
     def getTraceCollections(self):
         yield self.trace_collection
@@ -515,6 +532,11 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
         # Modules don't do this, pylint: disable=no-self-use
         return ()
 
+    @staticmethod
+    def getFunctionVariablesWithAutoReleases():
+        """Return the list of function variables that should be released at exit."""
+        return ()
+
     def getOutlineLocalVariables(self):
         outlines = self.getTraceCollection().getOutlineFunctions()
 
@@ -526,7 +548,7 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
         for outline in outlines:
             result.extend(outline.getUserLocalVariables())
 
-        return tuple(result)
+        return result
 
     def hasClosureVariable(self, variable):
         # Modules don't do this, pylint: disable=no-self-use,unused-argument
@@ -542,29 +564,21 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
                 outline.removeUserVariable(variable)
                 break
 
-    @staticmethod
-    def getFunctionLocalsScope():
-        """ Modules have no locals scope. """
-        return None
-
-    def getModuleDictScope(self):
-        return getLocalsDictHandle(self.module_dict_name)
+    def getLocalsScope(self):
+        return self.locals_scope
 
 
 class CompiledPythonPackage(CompiledPythonModule):
     kind = "COMPILED_PYTHON_PACKAGE"
 
-    def __init__(self, name, package_name, is_top, mode, future_spec, source_ref):
-        assert name, source_ref
-
+    def __init__(self, module_name, is_top, mode, future_spec, source_ref):
         CompiledPythonModule.__init__(
             self,
-            name         = name,
-            package_name = package_name,
-            is_top       = is_top,
-            mode         = mode,
-            future_spec  = future_spec,
-            source_ref   = source_ref
+            module_name=module_name,
+            is_top=is_top,
+            mode=mode,
+            future_spec=future_spec,
+            source_ref=source_ref,
         )
 
     def getOutputFilename(self):
@@ -580,53 +594,42 @@ class CompiledPythonPackage(CompiledPythonModule):
         return True
 
 
-def makeUncompiledPythonModule(module_name, filename, bytecode, is_package,
-                               user_provided, technical):
-    parts = module_name.rsplit('.', 1)
-    name = parts[-1]
-
-    package_name = parts[0] if len(parts) == 2 else None
+def makeUncompiledPythonModule(
+    module_name, filename, bytecode, is_package, user_provided, technical
+):
     source_ref = fromFilename(filename)
 
     if is_package:
         return UncompiledPythonPackage(
-            name          = name,
-            package_name  = package_name,
-            bytecode      = bytecode,
-            filename      = filename,
-            user_provided = user_provided,
-            technical     = technical,
-            source_ref    = source_ref
+            module_name=module_name,
+            bytecode=bytecode,
+            filename=filename,
+            user_provided=user_provided,
+            technical=technical,
+            source_ref=source_ref,
         )
     else:
         return UncompiledPythonModule(
-            name          = name,
-            package_name  = package_name,
-            bytecode      = bytecode,
-            filename      = filename,
-            user_provided = user_provided,
-            technical     = technical,
-            source_ref    = source_ref
+            module_name=module_name,
+            bytecode=bytecode,
+            filename=filename,
+            user_provided=user_provided,
+            technical=technical,
+            source_ref=source_ref,
         )
 
 
 class UncompiledPythonModule(PythonModuleBase):
-    """ Compiled Python Module
-
-    """
+    """Compiled Python Module"""
 
     kind = "UNCOMPILED_PYTHON_MODULE"
 
     __slots__ = "bytecode", "filename", "user_provided", "technical", "used_modules"
 
-    def __init__(self, name, package_name, bytecode, filename, user_provided,
-                 technical, source_ref):
-        PythonModuleBase.__init__(
-            self,
-            name         = name,
-            package_name = package_name,
-            source_ref   = source_ref
-        )
+    def __init__(
+        self, module_name, bytecode, filename, user_provided, technical, source_ref
+    ):
+        PythonModuleBase.__init__(self, module_name=module_name, source_ref=source_ref)
 
         self.bytecode = bytecode
         self.filename = filename
@@ -675,23 +678,22 @@ class PythonMainModule(CompiledPythonModule):
     kind = "PYTHON_MAIN_MODULE"
 
     def __init__(self, main_added, mode, future_spec, source_ref):
+        self.main_added = main_added
+
         CompiledPythonModule.__init__(
             self,
-            name         = "__main__",
-            package_name = None,
-            is_top       = True,
-            mode         = mode,
-            future_spec  = future_spec,
-            source_ref   = source_ref
+            module_name=ModuleName("__main__"),
+            is_top=True,
+            mode=mode,
+            future_spec=future_spec,
+            source_ref=source_ref,
         )
 
-        self.main_added = main_added
-
     def getDetails(self):
         return {
-            "filename"   : self.source_ref.getFilename(),
-            "main_added" : self.main_added,
-            "mode"       : self.mode
+            "filename": self.source_ref.getFilename(),
+            "main_added": self.main_added,
+            "mode": self.mode,
         }
 
     @classmethod
@@ -700,19 +702,22 @@ class PythonMainModule(CompiledPythonModule):
             future_spec = fromFlags(args["code_flags"])
 
         result = cls(
-            main_added  = args["main_added"] == "True",
-            mode        = args["mode"],
-            future_spec = future_spec,
-            source_ref  = source_ref
+            main_added=args["main_added"] == "True",
+            mode=args["mode"],
+            future_spec=future_spec,
+            source_ref=source_ref,
         )
 
         from nuitka.ModuleRegistry import addRootModule
+
         addRootModule(result)
 
         function_work = []
 
         for xml in args["functions"]:
-            _kind, node_class, func_args, source_ref = extractKindAndArgsFromXML(xml, source_ref)
+            _kind, node_class, func_args, source_ref = extractKindAndArgsFromXML(
+                xml, source_ref
+            )
 
             if "provider" in func_args:
                 func_args["provider"] = getOwnerFromCodeName(func_args["provider"])
@@ -720,38 +725,26 @@ class PythonMainModule(CompiledPythonModule):
                 func_args["provider"] = result
 
             if "flags" in args:
-                func_args["flags"] = set(func_args["flags"].split(','))
+                func_args["flags"] = set(func_args["flags"].split(","))
 
             if "doc" not in args:
                 func_args["doc"] = None
 
-            function = node_class.fromXML(
-                source_ref = source_ref,
-                **func_args
-            )
+            function = node_class.fromXML(source_ref=source_ref, **func_args)
 
             # Could do more checks for look up of body here, but so what...
-            function_work.append(
-                (function, iter(iter(xml).next()).next())
-            )
+            function_work.append((function, iter(iter(xml).next()).next()))
 
         for function, xml in function_work:
             function.setChild(
                 "body",
                 fromXML(
-                    provider   = function,
-                    xml        = xml,
-                    source_ref = function.getSourceReference()
-                )
+                    provider=function, xml=xml, source_ref=function.getSourceReference()
+                ),
             )
 
         result.setChild(
-            "body",
-            fromXML(
-                provider   = result,
-                xml        = args["body"][0],
-                source_ref = source_ref
-            )
+            "body", fromXML(provider=result, xml=args["body"][0], source_ref=source_ref)
         )
 
         return result
@@ -767,31 +760,6 @@ class PythonMainModule(CompiledPythonModule):
             return CompiledPythonModule.getOutputFilename(self)
 
 
-class PythonInternalModule(CompiledPythonModule):
-    kind = "PYTHON_INTERNAL_MODULE"
-
-    def __init__(self):
-        CompiledPythonModule.__init__(
-            self,
-            name         = "__internal__",
-            package_name = None,
-            is_top       = False,
-            mode         = "compiled",
-            source_ref   = SourceCodeReference.fromFilenameAndLine(
-                filename = "internal",
-                line     = 0
-            ),
-            future_spec  = FutureSpec()
-        )
-
-    @staticmethod
-    def isInternalModule():
-        return True
-
-    def getOutputFilename(self):
-        return "__internal"
-
-
 class PythonShlibModule(PythonModuleBase):
     kind = "PYTHON_SHLIB_MODULE"
 
@@ -799,19 +767,14 @@ class PythonShlibModule(PythonModuleBase):
 
     avoid_duplicates = set()
 
-    def __init__(self, name, package_name, source_ref):
-        PythonModuleBase.__init__(
-            self,
-            name         = name,
-            package_name = package_name,
-            source_ref   = source_ref
-        )
+    def __init__(self, module_name, source_ref):
+        PythonModuleBase.__init__(self, module_name=module_name, source_ref=source_ref)
 
         # That would be a mistake we just made.
         assert os.path.basename(source_ref.getFilename()) != "<frozen>"
 
         # That is too likely a bug.
-        assert name != "__main__"
+        assert module_name != "__main__"
 
         # Duplicates should be avoided by us caching elsewhere before creating
         # the object.
@@ -823,12 +786,6 @@ class PythonShlibModule(PythonModuleBase):
     def finalize(self):
         del self.used_modules
 
-    def getDetails(self):
-        return {
-            "name"         : self.name,
-            "package_name" : self.package_name
-        }
-
     def getFilename(self):
         return self.getSourceReference().getFilename()
 
@@ -842,58 +799,80 @@ class PythonShlibModule(PythonModuleBase):
         filename = os.path.basename(path)
         dirname = os.path.dirname(path)
 
-        return os.path.join(dirname, filename.split('.')[0]) + ".pyi"
+        return os.path.join(dirname, filename.split(".")[0]) + ".pyi"
 
     def _readPyPIFile(self):
         """ Read the .pyi file if present and scan for dependencies. """
 
+        # Complex stuff, pylint: disable=too-many-branches
+
         if self.used_modules is None:
             pyi_filename = self.getPyIFilename()
 
             if os.path.exists(pyi_filename):
                 pyi_deps = OrderedSet()
 
-                for line in open(pyi_filename):
+                # Flag signalling multiline import handling
+                in_import = False
+                in_import_part = ""
+
+                for line in getFileContentByLine(pyi_filename):
                     line = line.strip()
 
-                    if line.startswith("import "):
-                        imported = line[7:]
+                    if not in_import:
+                        if line.startswith("import "):
+                            imported = line[7:]
 
-                        pyi_deps.add(imported)
-                    elif line.startswith("from "):
-                        parts = line.split(None, 3)
-                        assert parts[0] == "from"
-                        assert parts[2] == "import"
+                            pyi_deps.add(imported)
+                        elif line.startswith("from "):
+                            parts = line.split(None, 3)
+                            assert parts[0] == "from"
+                            assert parts[2] == "import"
 
-                        if parts[1] == "typing":
-                            continue
+                            if parts[1] == "typing":
+                                continue
 
-                        pyi_deps.add(parts[1])
+                            pyi_deps.add(parts[1])
 
-                        imported = parts[3]
-                        if imported.startswith('('):
-                            # No multiline imports please
-                            assert imported.endswith(')')
-                            imported = imported[1:-1]
+                            imported = parts[3]
+                            if imported.startswith("("):
+                                # Handle multiline imports
+                                if not imported.endswith(")"):
+                                    in_import = True
+                                    imported = imported[1:]
+                                    in_import_part = parts[1]
+                                    assert in_import_part, (
+                                        "Multiline part in file %s cannot be empty"
+                                        % pyi_filename
+                                    )
+                                else:
+                                    in_import = False
+                                    imported = imported[1:-1]
+                                    assert imported
 
-                            assert imported
+                            if imported == "*":
+                                continue
 
-                        if imported == '*':
-                            continue
+                            for name in imported.split(","):
+                                if name:
+                                    name = name.strip()
+                                    pyi_deps.add(parts[1] + "." + name)
 
-                        for name in imported.split(','):
+                    else:  # In import
+                        imported = line
+                        if imported.endswith(")"):
+                            imported = imported[0:-1]
+                            in_import = False
+
+                        for name in imported.split(","):
                             name = name.strip()
-
-                            pyi_deps.add(parts[1] + '.' + name)
+                            if name:
+                                pyi_deps.add(in_import_part + "." + name)
 
                 if "typing" in pyi_deps:
                     pyi_deps.discard("typing")
 
-                self.used_modules = tuple(
-                    (pyi_dep, None)
-                    for pyi_dep in
-                    pyi_deps
-                )
+                self.used_modules = tuple((pyi_dep, None) for pyi_dep in pyi_deps)
             else:
                 self.used_modules = ()
 
