@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,124 +25,100 @@ Predicting the behavior of 'print' is not trivial at all, due to many special
 cases.
 """
 
-from .NodeBases import StatementChildrenHavingBase
+from .NodeBases import StatementChildHavingBase, StatementChildrenHavingBase
 from .NodeMakingHelpers import (
     makeStatementExpressionOnlyReplacementNode,
     makeStatementsSequenceReplacementNode,
-    wrapStatementWithSideEffects
+    wrapStatementWithSideEffects,
 )
 
 
 class StatementPrintValue(StatementChildrenHavingBase):
     kind = "STATEMENT_PRINT_VALUE"
 
-    named_children = (
-        "dest",
-        "value"
-    )
+    named_children = ("dest", "value")
+    getDestination = StatementChildrenHavingBase.childGetter("dest")
+    getValue = StatementChildrenHavingBase.childGetter("value")
+    setValue = StatementChildrenHavingBase.childSetter("value")
 
     def __init__(self, dest, value, source_ref):
         StatementChildrenHavingBase.__init__(
-            self,
-            values     = {
-                "value" : value,
-                "dest"  : dest
-            },
-            source_ref = source_ref
+            self, values={"value": value, "dest": dest}, source_ref=source_ref
         )
 
         assert value is not None
 
-    getDestination = StatementChildrenHavingBase.childGetter(
-        "dest"
-    )
-
-    getValue = StatementChildrenHavingBase.childGetter(
-        "value"
-    )
-    setValue = StatementChildrenHavingBase.childSetter(
-        "value"
-    )
-
     def computeStatement(self, trace_collection):
-        trace_collection.onExpression(
-            expression = self.getDestination(),
-            allow_none = True
+        dest = trace_collection.onExpression(
+            expression=self.getDestination(), allow_none=True
         )
-        dest = self.getDestination()
 
         if dest is not None and dest.mayRaiseException(BaseException):
-            trace_collection.onExceptionRaiseExit(
-                BaseException
-            )
+            trace_collection.onExceptionRaiseExit(BaseException)
 
         if dest is not None and dest.willRaiseException(BaseException):
             result = makeStatementExpressionOnlyReplacementNode(
-                expression = dest,
-                node       = self
+                expression=dest, node=self
             )
 
-            return result, "new_raise", """\
-Exception raise in 'print' statement destination converted to explicit raise."""
+            return (
+                result,
+                "new_raise",
+                """\
+Exception raise in 'print' statement destination converted to explicit raise.""",
+            )
 
-        trace_collection.onExpression(
-            expression = self.getValue()
-        )
-        value = self.getValue()
+        value = trace_collection.onExpression(expression=self.getValue())
 
         if value.mayRaiseException(BaseException):
-            trace_collection.onExceptionRaiseExit(
-                BaseException
-            )
+            trace_collection.onExceptionRaiseExit(BaseException)
 
         if value.willRaiseException(BaseException):
             if dest is not None:
                 result = wrapStatementWithSideEffects(
-                    new_node = makeStatementExpressionOnlyReplacementNode(
-                        expression = value,
-                        node       = self
+                    new_node=makeStatementExpressionOnlyReplacementNode(
+                        expression=value, node=self
                     ),
-                    old_node = dest
+                    old_node=dest,
                 )
             else:
                 result = makeStatementExpressionOnlyReplacementNode(
-                    expression = value,
-                    node       = self
+                    expression=value, node=self
                 )
 
-            return result, "new_raise", """\
-Exception raise in 'print' statement arguments converted to explicit raise."""
+            return (
+                result,
+                "new_raise",
+                """\
+Exception raise in 'print' statement arguments converted to explicit raise.""",
+            )
 
-        trace_collection.onExceptionRaiseExit(
-            BaseException
-        )
+        trace_collection.onExceptionRaiseExit(BaseException)
 
         if dest is None:
             if value.isExpressionSideEffects():
                 self.setValue(value.getExpression())
 
                 statements = [
-                    makeStatementExpressionOnlyReplacementNode(
-                        side_effect,
-                        self
-                    )
-                    for side_effect in
-                    value.getSideEffects()
+                    makeStatementExpressionOnlyReplacementNode(side_effect, self)
+                    for side_effect in value.getSideEffects()
                 ]
 
                 statements.append(self)
 
                 result = makeStatementsSequenceReplacementNode(
-                    statements = statements,
-                    node       = self,
+                    statements=statements, node=self
                 )
 
-                return result, "new_statements", """\
-Side effects printed item promoted to statements."""
+                return (
+                    result,
+                    "new_statements",
+                    """\
+Side effects printed item promoted to statements.""",
+                )
 
         if value.isCompileTimeConstant():
-            if not (value.isExpressionConstantRef() and \
-                    value.isUnicodeConstant()):
+            if not (value.isExpressionConstantRef() and value.isUnicodeConstant()):
                 new_value = value.getStrValue()
                 assert new_value is not None, value
 
@@ -155,51 +131,36 @@ Side effects printed item promoted to statements."""
         return True
 
 
-class StatementPrintNewline(StatementChildrenHavingBase):
+class StatementPrintNewline(StatementChildHavingBase):
     kind = "STATEMENT_PRINT_NEWLINE"
 
-    named_children = (
-        "dest",
-    )
+    named_child = "dest"
+    getDestination = StatementChildHavingBase.childGetter("dest")
 
     def __init__(self, dest, source_ref):
-        StatementChildrenHavingBase.__init__(
-            self,
-            values     = {
-                "dest" : dest
-            },
-            source_ref = source_ref
-        )
-
-    getDestination = StatementChildrenHavingBase.childGetter(
-        "dest"
-    )
+        StatementChildHavingBase.__init__(self, value=dest, source_ref=source_ref)
 
     def computeStatement(self, trace_collection):
-        # TODO: Reactivate below optimizations for prints.
-        trace_collection.onExpression(
-            expression = self.getDestination(),
-            allow_none = True
+        dest = trace_collection.onExpression(
+            expression=self.getDestination(), allow_none=True
         )
-        dest = self.getDestination()
 
         if dest is not None and dest.mayRaiseException(BaseException):
-            trace_collection.onExceptionRaiseExit(
-                BaseException
-            )
+            trace_collection.onExceptionRaiseExit(BaseException)
 
         if dest is not None and dest.willRaiseException(BaseException):
             result = makeStatementExpressionOnlyReplacementNode(
-                expression = dest,
-                node       = self
+                expression=dest, node=self
             )
 
-            return result, "new_raise", """\
-Exception raise in 'print' statement destination converted to explicit raise."""
+            return (
+                result,
+                "new_raise",
+                """\
+Exception raise in 'print' statement destination converted to explicit raise.""",
+            )
 
-        trace_collection.onExceptionRaiseExit(
-            BaseException
-        )
+        trace_collection.onExceptionRaiseExit(BaseException)
 
         return self, None, None
 
