@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -20,50 +20,13 @@
 """
 
 import os
-import pkgutil
-from logging import warning
 
 from nuitka import Options
+from nuitka.__past__ import basestring  # pylint: disable=I0021,redefined-builtin
 from nuitka.containers.oset import OrderedSet
 from nuitka.plugins.PluginBase import NuitkaPluginBase
 from nuitka.utils.FileOperations import getFileList, listDir
 
-known_data_files = {
-    # Key is the package name to trigger it
-    # Value is a tuple of 2 element tuples, thus trailing commas, where
-    # the target path can be specified (None is just default, i.e. the
-    # package directory) and the filename relative to the source package
-    # directory
-    "botocore": ((None, "cacert.pem"),),
-    "site": ((None, "orig-prefix.txt"),),
-    "nose.core": ((None, "usage.txt"),),
-    "scrapy": ((None, "VERSION"),),
-    "dask": (("", "dask.yaml"),),
-    "cairocffi": ((None, "VERSION"),),
-    "cairosvg": ((None, "VERSION"),),
-    "weasyprint": ((None, "VERSION"),),
-    "tinycss2": ((None, "VERSION"),),
-    "requests": (("certifi", "../certifi/cacert.pem"),),
-    "urllib3": (("certifi", "../certifi/cacert.pem"),),
-    "importlib_resources": ((None, "version.txt"),),
-    "moto": (
-        (None, "./ec2/resources/instance_types.json"),
-        (None, "./ec2/resources/amis.json"),
-    ),
-    "skimage": (
-        (None, "./io/_plugins/fits_plugin.ini"),
-        (None, "./io/_plugins/gdal_plugin.ini"),
-        (None, "./io/_plugins/gtk_plugin.ini"),
-        (None, "./io/_plugins/imageio_plugin.ini"),
-        (None, "./io/_plugins/imread_plugin.ini"),
-        (None, "./io/_plugins/matplotlib_plugin.ini"),
-        (None, "./io/_plugins/pil_plugin.ini"),
-        (None, "./io/_plugins/qt_plugin.ini"),
-        (None, "./io/_plugins/simpleitk_plugin.ini"),
-        (None, "./io/_plugins/tifffile_plugin.ini"),
-    ),
-}
-
 
 def _createEmptyDirText(filename):
     # We create the same content all the time, pylint: disable=unused-argument
@@ -76,9 +39,7 @@ def _createEmptyDirNone(filename):
 
 
 def remove_suffix(string, suffix):
-    """ Remove 'suffix' from 'string'.
-
-    """
+    """Remove 'suffix' from 'string'."""
     # Special case: if suffix is empty, string[:0] returns ''. So, test
     # for a non-empty suffix.
     if suffix and string.endswith(suffix):
@@ -88,13 +49,15 @@ def remove_suffix(string, suffix):
 
 
 def get_package_paths(package):
-    """ Return the path to the package.
+    """Return the path to the package.
 
     Args:
         package: (str) package name
     Returns:
         tuple: (prefix, prefix/package)
     """
+    import pkgutil
+
     loader = pkgutil.find_loader(package)
     if not loader:
         return "", ""
@@ -109,15 +72,7 @@ def get_package_paths(package):
     return pkg_base, pkg_dir
 
 
-generated_data_files = {
-    "Cryptodome.Util._raw_api": (
-        ("Cryptodome/Util", ".keep_dir.txt", _createEmptyDirText),
-    ),
-    "Crypto.Util._raw_api": (("Crypto/Util", ".keep_dir.txt", _createEmptyDirText),),
-}
-
-
-def _get_package_files(module, packages, folders_only):
+def _getPackageFiles(module, packages, folders_only):
     """Yield all (!) filenames in given package(s).
 
     Notes:
@@ -134,27 +89,35 @@ def _get_package_files(module, packages, folders_only):
         Tuples of paths (source, dest), if folders_only is False,
         else tuples (_createEmptyDirNone, dest).
     """
+
+    # TODO: Maybe use isinstance(basestring) for this
     if not hasattr(packages, "__getitem__"):  # so should be a string type
         packages = (packages,)
 
     file_list = []
     item_set = OrderedSet()
 
+    file_dirs = []
+
     for package in packages:
         pkg_base, pkg_dir = get_package_paths(package)  # read package folders
         if pkg_dir:
             filename_start = len(pkg_base)  # position of package name in dir
-            pkg_files = getFileList(pkg_dir)  # read out the filenames
+            # read out the filenames
+            pkg_files = getFileList(
+                pkg_dir, ignore_dirs=("__pycache__",), ignore_suffixes=(".pyc",)
+            )
+            file_dirs.append(pkg_dir)
             for f in pkg_files:
                 file_list.append((filename_start, f))  # append to file list
 
-    if file_list == []:  #  safeguard for unexpected cases
-        msg = "No files or folders found for '%s' in packages(s) '%s'." % (
+    if not file_list:  #  safeguard for unexpected cases
+        msg = "No files or folders found for '%s' in packages(s) '%r' (%r)." % (
             module.getFullName(),
-            str(packages),
+            packages,
+            file_dirs,
         )
-        warning(msg)
-        yield ()
+        NuitkaPluginDataFileCollector.warning(msg)
 
     for filename_start, f in file_list:  # re-read the collected filenames
         target = f[filename_start:]  # make part of name
@@ -167,7 +130,7 @@ def _get_package_files(module, packages, folders_only):
         yield f
 
 
-def _get_subdir_files(module, subdirs, folders_only):
+def _getSubDirectoryFiles(module, subdirs, folders_only):
     """Yield filenames in given subdirs of the module.
 
     Notes:
@@ -190,27 +153,32 @@ def _get_subdir_files(module, subdirs, folders_only):
     item_set = OrderedSet()
 
     if subdirs is None:
-        file_list = getFileList(module_folder)
-    elif type(subdirs) is str:
-        data_dir = os.path.join(module_folder, subdirs)
-        file_list = getFileList(data_dir)
+        data_dirs = [module_folder]
+    elif isinstance(subdirs, basestring):
+        data_dirs = [os.path.join(module_folder, subdirs)]
     else:
-        for subdir in subdirs:
-            data_dir = os.path.join(module_folder, subdir)
-            file_list.extend(getFileList(data_dir))
+        data_dirs = [os.path.join(module_folder, subdir) for subdir in subdirs]
 
-    if file_list == []:
-        msg = "No files or folders found for '%s' in subfolder(s) '%s'." % (
+    # Gather the full file list, probably makes no sense to include bytecode files
+    file_list = sum(
+        (
+            getFileList(
+                data_dir, ignore_dirs=("__pycache__",), ignore_suffixes=(".pyc",)
+            )
+            for data_dir in data_dirs
+        ),
+        [],
+    )
+
+    if not file_list:
+        msg = "No files or folders found for '%s' in subfolder(s) %r (%r)." % (
             module.getFullName(),
-            str(subdirs),
+            subdirs,
+            data_dirs,
         )
-        warning(msg)
-        yield ()
+        NuitkaPluginDataFileCollector.warning(msg)
 
     for f in file_list:
-        if "__pycache__" in f or f.endswith(".pyc"):  # probably makes no sense
-            continue
-
         target = f[filename_start:]
         if folders_only is False:
             item_set.add((f, target))
@@ -221,31 +189,78 @@ def _get_subdir_files(module, subdirs, folders_only):
         yield f
 
 
-# data files to be copied are contained in subfolders named as the second item
-# the 3rd item indicates whether to recreate toe folder structure only (True),
-# or indeed also copy the files.
-known_data_folders = {
-    "botocore": (_get_subdir_files, "data", False),
-    "boto3": (_get_subdir_files, "data", False),
-    "sklearn.datasets": (_get_subdir_files, ("data", "descr"), False),
-    "osgeo": (_get_subdir_files, "data", False),
-    "pyphen": (_get_subdir_files, "dictionaries", False),
-    "pendulum": (_get_subdir_files, "locales", True),  # folder structure only
-    "pytz": (_get_subdir_files, "zoneinfo", False),
-    "pytzdata": (_get_subdir_files, "zoneinfo", False),
-    "pywt": (_get_subdir_files, "data", False),
-    "skimage": (_get_subdir_files, "data", False),
-    "weasyprint": (_get_subdir_files, "css", False),
-    "xarray": (_get_subdir_files, "static", False),
-    "eventlet": (_get_package_files, ("dns",), False),  # copy other package source
-}
-
-
 class NuitkaPluginDataFileCollector(NuitkaPluginBase):
     plugin_name = "data-files"
 
-    @staticmethod
-    def isRelevant():
+    known_data_files = {
+        # Key is the package name to trigger it
+        # Value is a tuple of 2 element tuples, thus trailing commas, where
+        # the target path can be specified (None is just default, i.e. the
+        # package directory) and the filename relative to the source package
+        # directory
+        "botocore": ((None, "cacert.pem"),),
+        "site": ((None, "orig-prefix.txt"),),
+        "nose.core": ((None, "usage.txt"),),
+        "scrapy": ((None, "VERSION"),),
+        "dask": (("", "dask.yaml"),),
+        "cairocffi": ((None, "VERSION"),),
+        "cairosvg": ((None, "VERSION"),),
+        "weasyprint": ((None, "VERSION"),),
+        "tinycss2": ((None, "VERSION"),),
+        "certifi": ((None, "cacert.pem"),),
+        "importlib_resources": ((None, "version.txt"),),
+        "moto": (
+            (None, "ec2/resources/instance_types.json"),
+            (None, "ec2/resources/amis.json"),
+        ),
+        "skimage": (
+            (None, "io/_plugins/fits_plugin.ini"),
+            (None, "io/_plugins/gdal_plugin.ini"),
+            (None, "io/_plugins/gtk_plugin.ini"),
+            (None, "io/_plugins/imageio_plugin.ini"),
+            (None, "io/_plugins/imread_plugin.ini"),
+            (None, "io/_plugins/matplotlib_plugin.ini"),
+            (None, "io/_plugins/pil_plugin.ini"),
+            (None, "io/_plugins/qt_plugin.ini"),
+            (None, "io/_plugins/simpleitk_plugin.ini"),
+            (None, "io/_plugins/tifffile_plugin.ini"),
+        ),
+        "skimage.feature._orb_descriptor_positions": (
+            ("skimage/feature", "orb_descriptor_positions.txt"),
+        ),
+    }
+
+    # data files to be copied are contained in subfolders named as the second item
+    # the 3rd item indicates whether to recreate toe folder structure only (True),
+    # or indeed also copy the files.
+    known_data_folders = {
+        "botocore": (_getSubDirectoryFiles, "data", False),
+        "boto3": (_getSubDirectoryFiles, "data", False),
+        "sklearn.datasets": (_getSubDirectoryFiles, ("data", "descr"), False),
+        "osgeo": (_getSubDirectoryFiles, "data", False),
+        "pyphen": (_getSubDirectoryFiles, "dictionaries", False),
+        "pendulum": (_getSubDirectoryFiles, "locales", True),  # folder structure only
+        "pytz": (_getSubDirectoryFiles, "zoneinfo", False),
+        "pytzdata": (_getSubDirectoryFiles, "zoneinfo", False),
+        "pywt": (_getSubDirectoryFiles, "data", False),
+        "skimage": (_getSubDirectoryFiles, "data", False),
+        "weasyprint": (_getSubDirectoryFiles, "css", False),
+        "xarray": (_getSubDirectoryFiles, "static", False),
+        "eventlet": (_getPackageFiles, ("dns",), False),  # copy other package source
+        "gooey": (_getSubDirectoryFiles, ("languages", "images"), False),
+    }
+
+    generated_data_files = {
+        "Cryptodome.Util._raw_api": (
+            ("Cryptodome/Util", ".keep_dir.txt", _createEmptyDirText),
+        ),
+        "Crypto.Util._raw_api": (
+            ("Crypto/Util", ".keep_dir.txt", _createEmptyDirText),
+        ),
+    }
+
+    @classmethod
+    def isRelevant(cls):
         return Options.isStandaloneMode()
 
     @staticmethod
@@ -256,8 +271,8 @@ class NuitkaPluginDataFileCollector(NuitkaPluginBase):
         module_name = module.getFullName()
         module_folder = module.getCompileTimeDirectory()
 
-        if module_name in known_data_files:
-            for target_dir, filename in known_data_files[module_name]:
+        if module_name in self.known_data_files:
+            for target_dir, filename in self.known_data_files[module_name]:
                 source_path = os.path.join(module_folder, filename)
 
                 if os.path.isfile(source_path):
@@ -269,13 +284,13 @@ class NuitkaPluginDataFileCollector(NuitkaPluginBase):
                         os.path.normpath(os.path.join(target_dir, filename)),
                     )
 
-        if module_name in known_data_folders:
-            func, subdir, folders_only = known_data_folders[module_name]
+        if module_name in self.known_data_folders:
+            func, subdir, folders_only = self.known_data_folders[module_name]
             for item in func(module, subdir, folders_only):
                 yield item
 
-        if module_name in generated_data_files:
-            for target_dir, filename, func in generated_data_files[module_name]:
+        if module_name in self.generated_data_files:
+            for target_dir, filename, func in self.generated_data_files[module_name]:
                 if target_dir is None:
                     target_dir = module_name.replace(".", os.path.sep)
 
