@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -15,26 +15,32 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-import sys, os
+""" Reference counting tests.
+
+These contain functions that do specific things, where we have a suspect
+that references may be lost or corrupted. Executing them repeatedly and
+checking the reference count is how they are used.
+"""
+
+import os
+import sys
 
 # Find nuitka package relative to us.
 sys.path.insert(
     0,
     os.path.normpath(
-        os.path.join(
-            os.path.dirname(os.path.abspath(__file__)),
-            "..",
-            ".."
-        )
-    )
-)
-from nuitka.tools.testing.Common import (
-    executeReferenceChecked,
-    checkDebugPython
+        os.path.join(os.path.dirname(os.path.abspath(__file__)), "..", "..")
+    ),
 )
+from nuitka.tools.testing.Common import executeReferenceChecked
 
-checkDebugPython()
+# isort:start
 
+# Tests do all bad things:
+# pylint: disable=misplaced-bare-raise,raising-bad-type,reimported,undefined-variable
+# pylint: disable=broad-except,eval-used,redefined-outer-name,unused-argument,unused-variable
+# pylint: disable=attribute-defined-outside-init,bare-except,lost-exception,pointless-statement
+# pylint: disable=exec-used,global-statement,invalid-name,not-callable,super-init-not-called
 
 x = 17
 
@@ -43,14 +49,16 @@ x = 17
 def simpleFunction1():
     return 1
 
+
 # Do a bit of math with a local variable, assigning to its value and then doing
 # an overwrite of that, trying that math again. This should cover local access
 # a bit.
 def simpleFunction2():
-    y = 3 * x  # @UnusedVariable
+    y = 3 * x
     y = 3
 
-    return x*2*y
+    return x * 2 * y
+
 
 # A local function is being returned. This covers creation of local functions
 # and their release. No closure variables involved yet.
@@ -60,9 +68,10 @@ def simpleFunction3():
 
     return contained
 
+
 # Again, local function being return, but this time with local variable taken
 # as a closure. We use value from defaulted argument, so it cannot be replaced.
-def simpleFunction4(a = 1):
+def simpleFunction4(a=1):
     y = a
 
     def contained():
@@ -70,14 +79,16 @@ def simpleFunction4(a = 1):
 
     return contained
 
+
 # Default argument and building a list as local variables. Also return them,
 # so they are not optimized away.
-def simpleFunction5(a = 2):
+def simpleFunction5(a=2):
     c = 1
     f = [a, a + c]
 
     return c, f
 
+
 def simpleFunction6():
     for _b in range(6):
         pass
@@ -86,56 +97,68 @@ def simpleFunction6():
         pass
 
 
-def simpleFunction7(b = 1):
+def simpleFunction7(b=1):
     for _b in range(6):
         pass
 
+
 def simpleFunction8():
     c = []
     c.append(x)
 
-def simpleFunction9(a = 1*2):
+
+def simpleFunction9(a=1 * 2):
     if a == a:
         pass
 
+
 u = None
 
-def simpleFunction10(a = 1*2):
+
+def simpleFunction10(a=1 * 2):
     x = [u for u in range(8)]
 
+
 def simpleFunction11():
     f = 1
 
     while f < 8:
         f += 1
 
+
 v = None
 
+
 def simpleFunction12():
-    a = [(u,v) for (u,v) in zip(range(8),range(8))]
+    a = [(u, v) for (u, v) in zip(range(8), range(8))]
 
     return a
 
+
 def cond():
     return 1
 
-def simpleFunction13(a = 1*2):
+
+def simpleFunction13(a=1 * 2):
     pass
 
+
 def simpleFunction14p(x):
     try:
-        simpleFunction14p(1,1)
+        simpleFunction14p(1, 1)  # pylint: disable=too-many-function-args
     except TypeError as _e:
         pass
 
     try:
-        simpleFunction14p(1,1)
+        simpleFunction14p(1, 1)  # pylint: disable=too-many-function-args
     except TypeError:
         pass
 
+
 def simpleFunction14():
     simpleFunction14p(3)
 
+
 def simpleFunction15p(x):
     try:
         try:
@@ -148,21 +171,25 @@ def simpleFunction15p(x):
     except:
         pass
 
+
 def simpleFunction15():
     simpleFunction15p([1])
 
+
 def simpleFunction16():
     class EmptyClass:
         pass
 
     return EmptyClass
 
+
 def simpleFunction17():
     class EmptyObjectClass:
         pass
 
     return EmptyObjectClass()
 
+
 def simpleFunction18():
     closured = 1
 
@@ -175,8 +202,9 @@ def simpleFunction18():
 
     return NonEmptyClass(133, 135)
 
+
 def simpleFunction19():
-    lam = lambda l : l+1
+    lam = lambda l: l + 1
 
     return lam(9), lam
 
@@ -203,17 +231,21 @@ def simpleFunction21():
 
     return result, c
 
+
 def simpleFunction22():
     return True is False and False is not None
 
+
 def simpleFunction23():
     not 2
 
+
 def simpleFunction24p(x):
     pass
 
+
 def simpleFunction24():
-    simpleFunction24p(x = 3)
+    simpleFunction24p(x=3)
 
 
 def simpleFunction25():
@@ -223,7 +255,7 @@ def simpleFunction25():
     def inplace_adder(b):
         X.f += b
 
-    return inplace_adder(6**8)
+    return inplace_adder(6 ** 8)
 
 
 def simpleFunction26():
@@ -235,25 +267,28 @@ def simpleFunction26():
 
     return inplace_adder([1, 2])
 
+
 def simpleFunction27():
-    a = { 'g': 8 }
+    a = {"g": 8}
 
     def inplace_adder(b):
-        a[ 'g' ] += b
+        a["g"] += b
 
     return inplace_adder(3)
 
+
 def simpleFunction28():
-    a = { 'g': [8], 'h': 2 }
+    a = {"g": [8], "h": 2}
 
     def inplace_adder(b):
-        a[ 'g' ] += b
+        a["g"] += b
 
     return inplace_adder([3, 5])
 
 
 def simpleFunction29():
-    return '3' in '7'
+    return "3" in "7"
+
 
 def simpleFunction30():
     def generatorFunction():
@@ -261,6 +296,7 @@ def simpleFunction30():
         yield 2
         yield 3
 
+
 def simpleFunction31():
     def generatorFunction():
         yield 1
@@ -283,6 +319,7 @@ def simpleFunction32():
     gen = generatorFunction()
     next(gen)
 
+
 def simpleFunction33():
     def generatorFunction():
         a = 1
@@ -301,19 +338,21 @@ def simpleFunction34():
     except:
         pass
 
+
 def simpleFunction35():
     try:
-        raise ValueError(1,2,3)
+        raise ValueError(1, 2, 3)
     except:
         pass
 
 
 def simpleFunction36():
     try:
-        raise (TypeError, (3,x,x,x))
+        raise (TypeError, (3, x, x, x))
     except TypeError:
         pass
 
+
 def simpleFunction37():
     l = [1, 2, 3]
 
@@ -330,6 +369,7 @@ def simpleFunction38():
     class Parent(Base):
         pass
 
+
 def simpleFunction39():
     class Parent(object):
         pass
@@ -341,6 +381,7 @@ def simpleFunction40():
 
     myGenerator()
 
+
 def simpleFunction41():
     a = b = 2
 
@@ -352,6 +393,7 @@ def simpleFunction42():
 
     return a, b
 
+
 def simpleFunction43():
     class D:
         pass
@@ -360,6 +402,7 @@ def simpleFunction43():
 
     a.b = 1
 
+
 def simpleFunction47():
     def reraisy():
         def raisingFunction():
@@ -378,28 +421,34 @@ def simpleFunction47():
     except:
         pass
 
+
 def simpleFunction48():
     class BlockExceptions:
         def __enter__(self):
             pass
+
         def __exit__(self, exc, val, tb):
             return True
 
     with BlockExceptions():
         raise ValueError()
 
+
 template = "lala %s lala"
 
+
 def simpleFunction49():
     c = 3
     d = 4
 
-    a = x, y = b, e = (c,d)
+    a = x, y = b, e = (c, d)
 
     return a, y, b, e
 
+
 b = range(10)
 
+
 def simpleFunction50():
     def getF():
         def f():
@@ -413,32 +462,36 @@ def simpleFunction50():
     for x in range(2):
         _r = list(f())
 
+
 def simpleFunction51():
-    g = ( x for x in range(9) )
+    g = (x for x in range(9))
 
     try:
         g.throw(ValueError, 9)
     except ValueError as _e:
         pass
 
+
 def simpleFunction52():
-    g = ( x for x in range(9) )
+    g = (x for x in range(9))
 
     try:
         g.throw(ValueError(9))
     except ValueError as _e:
         pass
 
+
 def simpleFunction53():
-    g = ( x for x in range(9) )
+    g = (x for x in range(9))
 
     try:
         g.send(9)
     except TypeError as _e:
         pass
 
+
 def simpleFunction54():
-    g = ( x for x in range(9) )
+    g = (x for x in range(9))
     next(g)
 
     try:
@@ -448,34 +501,39 @@ def simpleFunction54():
 
 
 def simpleFunction55():
-    g = ( x for x in range(9) )
+    g = (x for x in range(9))
 
     try:
         g.close()
     except ValueError as _e:
         pass
 
+
 def simpleFunction56():
-    def f():
-        f()
+    """ Throw into finished generator. """
+    g = (x for x in range(9))
+
+    list(g)
 
     try:
-        f()
-    except RuntimeError:
+        g.throw(ValueError(9))
+    except ValueError as _e:
         pass
 
-def simpleFunction57():
+
+def simpleFunction60():
     x = 1
     y = 2
 
-    def f(a = x, b = y):
+    def f(a=x, b=y):
         return a, b
 
     f()
     f(2)
-    f(3,4)
+    f(3, 4)
 
-def simpleFunction58():
+
+def simpleFunction61():
     a = 3
     b = 5
 
@@ -487,7 +545,7 @@ def simpleFunction58():
         a / b
 
 
-def simpleFunction59():
+def simpleFunction62():
     a = 3
     b = 5
 
@@ -506,6 +564,7 @@ class X:
 
         raise ValueError(1)
 
+
 def simpleFunction63():
     def superUser():
         X()
@@ -515,23 +574,22 @@ def simpleFunction63():
     except Exception:
         pass
 
+
 def simpleFunction64():
     x = 2
-    y = 3  # @UnusedVariable
+    y = 3
     z = eval("x * y")
 
     return z
 
+
 def simpleFunction65():
     import array
 
-    a = array.array('b', b"")
+    a = array.array("b", b"")
     assert a == eval(repr(a), {"array": array.array})
 
-    d = {
-        'x' : 2,
-        'y' : 3
-    }
+    d = {"x": 2, "y": 3}
 
     z = eval(repr(d), d)
 
@@ -540,8 +598,10 @@ def simpleFunction65():
 
 def simpleFunction66():
     import types
+
     return type(simpleFunction65) == types.FunctionType
 
+
 def simpleFunction67():
     length = 100000
     pattern = "1234567890\00\01\02\03\04\05\06"
@@ -551,55 +611,62 @@ def simpleFunction67():
 
     return teststring
 
+
 def simpleFunction68():
     from random import randrange
+
     x = randrange(18)
 
+
 def simpleFunction69():
-    pools = [tuple() ]
+    pools = [tuple()]
     g = ((len(pool) == 0,) for pool in pools)
     next(g)
 
+
 def simpleFunction70():
     def gen():
         try:
-            undefined_yyy  # @UndefinedVariable
+            undefined_yyy
         except Exception:
             pass
 
         yield sys.exc_info()
 
     try:
-        undefined_xxx  # @UndefinedVariable
+        undefined_xxx
     except Exception:
         return list(gen())
 
+
 def simpleFunction71():
     try:
-        undefined_global  # @UndefinedVariable
+        undefined_global
     except Exception:
         try:
             try:
                 raise
             finally:
-                undefined_global  # @UndefinedVariable
+                undefined_global
         except Exception:
             pass
 
+
 def simpleFunction72():
     try:
         for _i in range(10):
             try:
-                undefined_global  # @UndefinedVariable
+                undefined_global
             finally:
                 break
     except Exception:
         pass
 
+
 def simpleFunction73():
     for _i in range(10):
         try:
-            undefined_global  # @UndefinedVariable
+            undefined_global
         finally:
             return 7
 
@@ -609,6 +676,7 @@ def simpleFunction74():
 
     return os
 
+
 def simpleFunction75():
     def raising_gen():
         try:
@@ -624,10 +692,12 @@ def simpleFunction75():
     except RuntimeError:
         pass
 
+
 def simpleFunction76():
     class MyException(Exception):
         def __init__(self, obj):
             self.obj = obj
+
     class MyObj:
         pass
 
@@ -645,19 +715,21 @@ def simpleFunction76():
         except MyException:
             pass
 
-def simpleFunction77():
-    class weirdstr(str):
-        def __getitem__(self, index):
-            return weirdstr(str.__getitem__(self, index))
 
-    (weirdstr("1234"))
-    # filter(lambda x: x>="33", weirdstr("1234"))
+class weirdstr(str):
+    def __getitem__(self, index):
+        return weirdstr(str.__getitem__(self, index))
+
+
+def simpleFunction77():
+    return filter(lambda x: x >= "33", weirdstr("1234"))
 
 
 def simpleFunction78():
     a = "x = 2"
     exec(a)
 
+
 def simpleFunction79():
     "some doc"
 
@@ -670,6 +742,7 @@ def simpleFunction80():
 
     del simpleFunction80.__doc__
 
+
 def simpleFunction81():
     def f():
         yield 1
@@ -678,6 +751,7 @@ def simpleFunction81():
     j = 1
     x = list(f())
 
+
 def simpleFunction82():
     def f():
         yield 1
@@ -686,137 +760,130 @@ def simpleFunction82():
     j = 1
     x = f.__doc__
 
+
 def simpleFunction83():
     x = list(range(7))
     x[2] = 5
 
     j = 3
-    x += [h*2 for h in range(j)]
+    x += [h * 2 for h in range(j)]
+
 
 def simpleFunction84():
     x = tuple(range(7))
 
     j = 3
-    x += tuple([h*2 for h in range(j)])
+    x += tuple([h * 2 for h in range(j)])
+
 
 def simpleFunction85():
     x = list(range(7))
     x[2] = 3
     x *= 2
 
+
 def simpleFunction86():
     x = "something"
     x += ""
 
+
 def simpleFunction87():
     x = 7
     x += 2000
 
+
+class C:
+    def f(self):
+        pass
+
+    def __iadd__(self, other):
+        return self
+
+
 def simpleFunction88():
-    class C:
-        def __iadd__(self, other):
-            return self
 
     x = C()
     x += C()
 
+
 def simpleFunction89():
-    x = [1,2]
-    x += [3,4]
+    x = [1, 2]
+    x += [3, 4]
+
 
 def anyArgs(*args, **kw):
     pass
 
+
 def simpleFunction90():
-    some_tuple = (
-        simpleFunction89,
-        simpleFunction89,
-        simpleFunction89,
-    )
+    some_tuple = (simpleFunction89, simpleFunction89, simpleFunction89)
 
     anyArgs(*some_tuple)
 
+
 def simpleFunction91():
-    some_dict = {
-        'a' : simpleFunction90,
-    }
+    some_dict = {"a": simpleFunction90}
 
     anyArgs(**some_dict)
 
-def simpleFunction92():
-    some_tuple = (
-        simpleFunction89,
-    )
 
-    some_dict = {
-        'a' : simpleFunction90,
-    }
+def simpleFunction92():
+    some_tuple = (simpleFunction89,)
+
+    some_dict = {"a": simpleFunction90}
 
     anyArgs(*some_tuple, **some_dict)
 
-def simpleFunction93():
-    some_tuple = (
-        simpleFunction89,
-    )
 
-    some_dict = {
-        'a' : simpleFunction90,
-    }
+def simpleFunction93():
+    some_tuple = (simpleFunction89,)
+
+    some_dict = {"a": simpleFunction90}
 
     anyArgs(some_tuple, *some_tuple, **some_dict)
 
+
 def simpleFunction94():
-    some_tuple = (
-        simpleFunction89,
-    )
+    some_tuple = (simpleFunction89,)
 
-    some_dict = {
-        'a' : simpleFunction90,
-    }
+    some_dict = {"a": simpleFunction90}
+
+    anyArgs(*some_tuple, b=some_dict, **some_dict)
 
-    anyArgs(*some_tuple, b = some_dict, **some_dict)
 
 def simpleFunction95():
-    some_tuple = (
-        simpleFunction89,
-    )
+    some_tuple = (simpleFunction89,)
 
-    some_dict = {
-        'a' : simpleFunction90,
-    }
+    some_dict = {"a": simpleFunction90}
+
+    anyArgs(some_tuple, *some_tuple, b=some_dict, **some_dict)
 
-    anyArgs(some_tuple, *some_tuple, b = some_dict, **some_dict)
 
 def simpleFunction96():
-    some_tuple = (
-        simpleFunction89,
-    )
+    some_tuple = (simpleFunction89,)
 
     anyArgs(some_tuple, *some_tuple)
 
+
 # Complex call with dictionary and key arguments only.
 def simpleFunction97():
-    some_dict = {
-        'a' : simpleFunction90,
-        'd' : simpleFunction91
-    }
+    some_dict = {"a": simpleFunction90, "d": simpleFunction91}
+
+    anyArgs(b=some_dict, c=1, **some_dict)
 
-    anyArgs(b = some_dict, c = 1, **some_dict)
 
 def simpleFunction98():
-    some_tuple = (
-        simpleFunction89,
-    )
+    some_tuple = (simpleFunction89,)
+
+    anyArgs(*some_tuple, b=some_tuple)
 
-    anyArgs(*some_tuple, b = some_tuple)
 
 def simpleFunction99():
-    some_dict = {
-        'a' : simpleFunction90,
-    }
+    some_dict = {"a": simpleFunction90}
 
     anyArgs(some_dict, **some_dict)
 
+
 def simpleFunction100():
     def h(f):
         def g():
@@ -829,39 +896,47 @@ def simpleFunction100():
 
     h(f)
 
+
 def simpleFunction101():
     def orMaking(a, b):
         x = "axa"
         x += a or b
 
+    orMaking("x", "")
 
-    orMaking('x', "")
 
 ####################################
 
+
 class SomeClassWithAttributeAccess(object):
     READING = 1
 
     def use(self):
         return self.READING
 
+
 def simpleFunction102():
     SomeClassWithAttributeAccess().use()
     SomeClassWithAttributeAccess().use()
 
+
 ####################################
 
+
 def getInt():
     return 3
 
+
 def simpleFunction103():
     try:
         raise getInt()
     except TypeError:
         pass
 
+
 ####################################
 
+
 class ClassWithGeneratorMethod:
     def generator_method(self):
         yield self
@@ -874,6 +949,7 @@ def simpleFunction104():
 def simpleFunction105():
     """ Delete a started generator, not properly closing it before releasing.
     """
+
     def generator():
         yield 1
         yield 2
@@ -883,49 +959,58 @@ def simpleFunction105():
 
     del g
 
+
 def simpleFunction106():
     # Call a PyCFunction with a single argument.
     return sys.getsizeof(type)
 
+
 def simpleFunction107():
     # Call a PyCFunction with a single argument.
     return sum(i for i in range(x))
 
+
 def simpleFunction108():
     # Call a PyCFunction with a single argument.
     return sum((i for i in range(x)), 17)
 
+
 def simpleFunction109():
     # Call a PyCFunction that looks like a method call.
     sys.exc_info()
 
+
 def simpleFunction110():
     def my_open(*args, **kwargs):
-        return(args, kwargs)
+        return (args, kwargs)
 
     orig_open = __builtins__.open
     __builtins__.open = my_open
 
-    open("me", buffering = True)
+    open("me", buffering=True)
     __builtins__.open = orig_open
 
+
 ####################################
 
-u = u'__name__'
+u = u"__name__"
+
 
 def simpleFunction111():
     return getattr(simpleFunction111, u)
 
+
 ####################################
 
+
 def simpleFunction112():
     TESTFN = "tmp.txt"
     import codecs
 
     try:
-        with open(TESTFN, 'wb') as out_file:
-            out_file.write(b'\xa1')
-        f = codecs.open(TESTFN, encoding='cp949')
+        with open(TESTFN, "wb") as out_file:
+            out_file.write(b"\xa1")
+        f = codecs.open(TESTFN, encoding="cp949")
         f.read(2)
     except UnicodeDecodeError:
         pass
@@ -942,16 +1027,20 @@ def simpleFunction112():
 
 ####################################
 
+
 def simpleFunction113():
     class A(object):
         pass
+
     a = A()
     a.a = a
 
     return a
 
+
 l = []
 
+
 def simpleFunction114():
     global l
     l += ["something"]
@@ -960,20 +1049,24 @@ def simpleFunction114():
     del l[:]
 
 
-i = 2**16+1
+i = 2 ** 16 + 1
+
 
 def simpleFunction115():
     global i
     i += 1
 
+
 t = tuple(range(259))
 
+
 def simpleFunction116():
     global t
     t += (2, 3)
 
     t = tuple(range(259))
 
+
 def simpleFunction117():
     # Operation tuple+object, error case.
     try:
@@ -989,16 +1082,20 @@ def simpleFunction118():
     except TypeError:
         pass
 
+
 t2 = tuple(range(9))
 
+
 def simpleFunction119():
     # Operation tuple+object no error case.
     return tuple(t) + t2
 
+
 def simpleFunction120():
     # Operation object+tuple no error case.
     return t2 + tuple(t)
 
+
 def simpleFunction121():
     # Operation tuple+tuple
     return tuple(t2) + tuple(t)
@@ -1019,16 +1116,20 @@ def simpleFunction123():
     except TypeError:
         pass
 
+
 l2 = list(range(9))
 
+
 def simpleFunction124():
     # Operation list+object no error case.
     return list(t) + l2
 
+
 def simpleFunction125():
     # Operation object+list no error case.
     return l2 + list(t)
 
+
 def simpleFunction126():
     # Operation tuple+tuple
     return list(l2) + list(t)
@@ -1041,14 +1142,17 @@ class TupleWithSlots(tuple):
     def __radd__(self, other):
         return 42
 
+
 def simpleFunction127():
     # Operation tuple+object with add slot.
     return tuple(t) + TupleWithSlots()
 
+
 def simpleFunction128():
     # Operation object+tuple with add slot.
     return TupleWithSlots() + tuple(t)
 
+
 class ListWithSlots(list):
     def __add__(self, other):
         return 42
@@ -1056,15 +1160,27 @@ class ListWithSlots(list):
     def __radd__(self, other):
         return 42
 
+
 def simpleFunction129():
     # Operation list+object with add slot.
     return list(t) + ListWithSlots()
 
+
 def simpleFunction130():
     # Operation list+object with add slot.
     return ListWithSlots() + list(t)
 
 
+def simpleFunction131():
+    try:
+        C().f.__reduce__()
+    except Exception as e:
+        assert sys.version_info < (3, 4)
+
+
+def simpleFunction132():
+    C().f.__reduce_ex__(5)
+
 
 ####################################
 
@@ -1075,10 +1191,10 @@ tests_stderr = (63,)
 tests_skipped = {}
 
 result = executeReferenceChecked(
-    prefix        = "simpleFunction",
-    names         = globals(),
-    tests_skipped = tests_skipped,
-    tests_stderr  = tests_stderr
+    prefix="simpleFunction",
+    names=globals(),
+    tests_skipped=tests_skipped,
+    tests_stderr=tests_stderr,
 )
 
 sys.exit(0 if result else 1)
