@@ -38,6 +38,9 @@ sys.path.insert(
 
 # isort:start
 
+# Tests do all bad things:
+# pylint: disable=not-an-iterable
+
 import asyncio
 import types
 
@@ -195,6 +198,23 @@ def simpleFunction10():
     asyncio.get_event_loop().run_until_complete(run())
 
 
+def nosimpleFunction11():
+    async def someCoroutine():
+        return 10
+
+    coro = someCoroutine()
+
+    def someGenerator():
+        yield from coro
+
+    try:
+        list(someGenerator())
+    except TypeError:
+        pass
+
+    coro.close()
+
+
 # These need stderr to be wrapped.
 tests_stderr = ()
 
