@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -17,6 +17,8 @@
 #
 import pkgutil
 
-__version__ = pkgutil.get_data(__package__ or __name__, 'DATA_FILE.txt').decode('ascii').strip()
+__version__ = (
+    pkgutil.get_data(__package__ or __name__, "DATA_FILE.txt").decode("ascii").strip()
+)
 
 print(__version__)
