@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
 #     parts from me are licensed as below. It is at least Free Software where
@@ -17,8 +17,18 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
+""" Lxml standalone basic test.
+
+"""
+
+from __future__ import print_function
+
 import lxml.etree
 
+# nuitka-skip-unless-imports: lxml.etree
+
 tree = lxml.etree.fromstring("<root>value</root>")
 assert tree.tag == "root"
 assert tree.text == "value"
+
+print("OK")
